USE Weather

DELIMITER $$

CREATE PROCEDURE Barometer_HL_UPD(IN nRecID BIGINT, NewPressure DECIMAL(8,3), NewF CHAR(1))
MODIFIES SQL DATA
BEGIN

DECLARE INSdatetime DATETIME;
DECLARE INSdate DATE;
DECLARE INSymd INTEGER(8);
DECLARE INSym INTEGER(6);
DECLARE INSy SMALLINT(4);
DECLARE INSm SMALLINT(2);
DECLARE INSvpID TINYINT;

DECLARE maxPressure DECIMAL(8,3);
DECLARE minPressure DECIMAL(8,3);

SELECT bDateTime, bDate, YearMonthDay, YearMonth, bYear, bMonth, vpID INTO INSdatetime, INSdate, INSymd, INSym, INSy, INSm, INSvpID FROM DT USE INDEX (RecID) WHERE RecID = nRecID;

SELECT BarometerHigh, BarometerLow INTO maxPressure, minPressure FROM DHL USE INDEX (PRIMARY) WHERE YearMonthDay = INSymd && vpID = INSvpID;

IF NewPressure IS NOT NULL && (NewPressure > maxPressure || maxPressure IS NULL) THEN
    UPDATE DHL SET BarometerHigh = NewPressure, BarometerHighDT = INSdatetime, F = NewF WHERE YearMonthDay = INSymd && vpID = INSvpID;
END IF;

IF NewPressure IS NOT NULL && NewPressure > 0 && (NewPressure < minPressure || minPressure IS NULL) THEN
    UPDATE DHL SET BarometerLow = NewPressure, BarometerLowDT = INSdatetime, F = NewF WHERE YearMonthDay = INSymd && vpID = INSvpID;
END IF;

SELECT BarometerHigh, BarometerLow INTO maxPressure, minPressure FROM MHL USE INDEX (PRIMARY) WHERE YearMonth = INSym && vpID = INSvpID;

IF NewPressure IS NOT NULL && (NewPressure > maxPressure || maxPressure IS NULL) THEN
    UPDATE MHL SET BarometerHigh = NewPressure, BarometerHighDT = INSdatetime, F = NewF WHERE YearMonth = INSym && vpID = INSvpID;
END IF;

IF NewPressure IS NOT NULL && NewPressure > 0 && (NewPressure < minPressure || minPressure IS NULL) THEN
    UPDATE MHL SET BarometerLow = NewPressure, BarometerLowDT = INSdatetime, F = NewF WHERE YearMonth = INSym && vpID = INSvpID;
END IF;

SELECT BarometerHigh, BarometerLow INTO maxPressure, minPressure FROM YHL USE INDEX (PRIMARY) WHERE bYear = INSy && vpID = INSvpID;

IF NewPressure IS NOT NULL && (NewPressure > maxPressure || maxPressure IS NULL) THEN
    UPDATE YHL SET BarometerHigh = NewPressure, BarometerHighDT = INSdatetime, F = NewF WHERE bYear = INSy && vpID = INSvpID;
END IF;

IF NewPressure IS NOT NULL && NewPressure > 0 && (NewPressure < minPressure || minPressure IS NULL) THEN
    UPDATE YHL SET BarometerLow = NewPressure, BarometerLowDT = INSdatetime, F = NewF WHERE bYear = INSy && vpID = INSvpID;
END IF;

END$$

CREATE PROCEDURE Barometer_HL_SET(IN nRecID BIGINT)
MODIFIES SQL DATA
BEGIN

DECLARE INSdate DATE;
DECLARE INSymd INTEGER(8);
DECLARE INSym INTEGER(6);
DECLARE INSy SMALLINT(4);
DECLARE INSm SMALLINT(2);
DECLARE INSvpID TINYINT;

DECLARE maxBaroPress DECIMAL(8,3);
DECLARE maxBaroPressDT DATETIME;
DECLARE minBaroPress DECIMAL(8,3);
DECLARE minBaroPressDT DATETIME;

DECLARE curF CHAR(1);

SELECT bDate, YearMonthDay, YearMonth, bYear, bMonth, vpID INTO INSdate, INSymd, INSym, INSy, INSm, INSvpID FROM DT USE INDEX (RecID) WHERE RecID = nRecID;

SELECT F INTO curF FROM Barometer USE INDEX (PRIMARY) WHERE RecID = nRecID;

SELECT max(t1.Pressure), min(t1.Pressure) INTO maxBaroPress, minBaroPress FROM Barometer AS t1 INNER JOIN DT AS t2 USE INDEX (YMDvpID) USING (RecID) WHERE t2.YearMonthDay = INSymd && t2.vpID = INSvpID;

SELECT t2.bDateTime INTO maxBaroPressDT FROM Barometer AS t1 INNER JOIN DT AS t2 USE INDEX (YMDvpID) USING (RecID) WHERE t1.Pressure = maxBaroPress && t2.YearMonthDay = INSymd && t2.vpID = INSvpID ORDER BY t2.bDateTime LIMIT 1;

SELECT t2.bDateTime INTO minBaroPressDT FROM Barometer AS t1 INNER JOIN DT AS t2 USE INDEX (YMDvpID) USING (RecID) WHERE t1.Pressure = minBaroPress && t2.YearMonthDay = INSymd && t2.vpID = INSvpID ORDER BY t2.bDateTime LIMIT 1;

UPDATE DHL SET BarometerHigh = maxBaroPress, BarometerHighDT = maxBaroPressDT, BarometerLow = minBaroPress, BarometerLowDT = minBaroPressDT, F = curF WHERE YearMonthDay = INSymd && vpID = INSvpID;

SELECT max(BarometerHigh), min(BarometerLow) INTO maxBaroPress, minBaroPress FROM DHL USE INDEX (YMvpID) WHERE YearMonth = INSym && vpID = INSvpID;

SELECT BarometerHighDT INTO maxBaroPressDT FROM DHL USE INDEX (YMvpID) WHERE BarometerHigh = maxBaroPress && YearMonth = INSym && vpID = INSvpID LIMIT 1;

SELECT BarometerLowDT INTO minBaroPressDT FROM DHL USE INDEX (YMvpID) WHERE BarometerLow = minBaroPress && YearMonth = INSym && vpID = INSvpID LIMIT 1;

UPDATE MHL SET BarometerHigh = maxBaroPress, BarometerHighDT = maxBaroPressDT, BarometerLow = minBaroPress, BarometerLowDT = minBaroPressDT, F = curF WHERE YearMonth = INSym && vpID = INSvpID;

SELECT max(BarometerHigh), min(BarometerLow) INTO maxBaroPress, minBaroPress FROM MHL USE INDEX (YvpID) WHERE bYear = INSy && vpID = INSvpID;

SELECT BarometerHighDT INTO maxBaroPressDT FROM MHL USE INDEX (YvpID) WHERE BarometerHigh = maxBaroPress && bYear = INSy && vpID = INSvpID LIMIT 1;

SELECT BarometerLowDT INTO minBaroPressDT FROM MHL USE INDEX (YvpID) WHERE BarometerLow = minBaroPress && bYear = INSy && vpID = INSvpID LIMIT 1;

UPDATE YHL SET BarometerHigh = maxBaroPress, BarometerHighDT = maxBaroPressDT, BarometerLow = minBaroPress, BarometerLowDT = minBaroPressDT, F = curF WHERE bYear = INSy && vpID = INSvpID;

END$$

CREATE PROCEDURE Barometer_HL_RCL(IN nYMD INTEGER(8), nvpID TINYINT)
MODIFIES SQL DATA
BEGIN

DECLARE dRecID BIGINT;
DECLARE INSdate DATE;
DECLARE INSymd INTEGER(8);
DECLARE INSym INTEGER(6);
DECLARE INSy SMALLINT(4);
DECLARE INSm SMALLINT(2);
DECLARE INSd SMALLINT(2);
DECLARE dYMD INTEGER(8);

SELECT RecID, bDate, YearMonthDay, YearMonth, bYear, bMonth, bDay INTO dRecID, INSdate, INSymd, INSym, INSy, INSm, INSd FROM DT USE INDEX (YMDvpID) WHERE YearMonthDay = nYMD && vpID = nvpID LIMIT 1;

IF dRecID IS NOT NULL && dRecID > 0 THEN
    SELECT YearMonthDay INTO dYMD FROM YMD USE INDEX (PRIMARY) where YearMonthDay = nYMD && vpID = nvpID LIMIT 1;
    IF dYMD IS NULL THEN
        INSERT IGNORE INTO YMD (YearMonthDay, YearMonth, bDate, bYear, bMonth, bDay, vpID) VALUES (INSymd, INSym, INSdate, INSy, INSm, INSd, nvpID);
    END IF;
    CALL Barometer_HL_SET(dRecID);
END IF;

END$$

CREATE TRIGGER Barometer_INS AFTER INSERT ON Barometer
FOR EACH ROW
BEGIN
CALL Barometer_HL_UPD(NEW.RecID,NEW.Pressure,NEW.F);
END$$

CREATE TRIGGER Barometer_UPD AFTER UPDATE ON Barometer
FOR EACH ROW
BEGIN
CALL Barometer_HL_UPD(NEW.RecID,NEW.Pressure,NEW.F);
END$$

CREATE TRIGGER Barometer_DEL AFTER DELETE ON Barometer
FOR EACH ROW
BEGIN
CALL Barometer_HL_SET(OLD.RecID);
END$$


DELIMITER ;


