USE Weather

DELIMITER $$

CREATE PROCEDURE Hourly_HL_UPD(nRecID BIGINT, NewBarometer DECIMAL(8,3), NewDewPoint SMALLINT, NewEvpT DECIMAL(6,3), NewHeatIndex SMALLINT, NewInTemp SMALLINT, NewOutTemp SMALLINT, NewInHum SMALLINT, NewOutHum SMALLINT, NewRainAmount DECIMAL(6,2), NewRainRate DECIMAL(7,2), NewSolarRadAvg SMALLINT, NewTHSWIndex SMALLINT, NewWindChill SMALLINT, NewWindDir SMALLINT, NewWindGust SMALLINT, NewWindSpeed SMALLINT, NewF CHAR(1))
MODIFIES SQL DATA
BEGIN

DECLARE INSdatetime DATETIME;
DECLARE INSdate DATE;
DECLARE INSymd INTEGER(8);
DECLARE INSym INTEGER(6);
DECLARE INSy SMALLINT(4);
DECLARE INSm SMALLINT(2);
DECLARE INSvpID TINYINT;

DECLARE maxDewPoint SMALLINT;
DECLARE minDewPoint SMALLINT;
DECLARE maxTHSWIndex SMALLINT;
DECLARE sumEvpT DECIMAL(6,3);
DECLARE NEWsumEvpT DECIMAL(6,3);
DECLARE maxEvpT DECIMAL(6,3);
DECLARE maxOutTemp SMALLINT;
DECLARE minOutTemp SMALLINT;
DECLARE maxInTemp SMALLINT;
DECLARE minInTemp SMALLINT;
DECLARE maxOutHum SMALLINT;
DECLARE minOutHum SMALLINT;
DECLARE maxInHum SMALLINT;
DECLARE minInHum SMALLINT;
DECLARE maxBaro DECIMAL(8,3);
DECLARE minBaro DECIMAL(8,3);
DECLARE maxRain DECIMAL(6,2);
DECLARE maxRainRate DECIMAL(7,2);
DECLARE maxWindSpeed SMALLINT;
DECLARE maxWindDir SMALLINT;
DECLARE maxWindGust SMALLINT;
DECLARE minWindChill SMALLINT;
DECLARE maxHeatIndex SMALLINT;
DECLARE maxSolarRadAvg SMALLINT;

SELECT bDateTime, bDate, YearMonthDay, YearMonth, bYear, bMonth, vpID INTO INSdatetime, INSdate, INSymd, INSym, INSy, INSm, INSvpID FROM DT USE INDEX (RecID) WHERE RecID = nRecID;

SELECT HourlyDewPointHigh, HourlyDewPointLow, HourlyTHSWIndex, HourlyTotalET, HourlyETHigh, HourlyOutTempHigh, HourlyOutTempLow, HourlyInTempHigh, HourlyInTempLow, HourlyOutHumHigh, HourlyOutHumLow, HourlyInHumHigh, HourlyInHumLow, HourlyBarometerHigh, HourlyBarometerLow, HourlyRainAmount, HourlyRainRate, HourlyWindSpeedAvg, HourlyWindDirAvg, HourlyWindSpeedHigh, HourlyWindChill, HourlyHeatIndex, HourlySolarRad INTO maxDewPoint, minDewPoint, maxTHSWIndex, sumEvpT, maxEvpT, maxOutTemp, minOutTemp, maxInTemp, minInTemp, maxOutHum, minOutHum, maxInHum, minInHum, maxBaro, minBaro, maxRain, maxRainRate, maxWindSpeed, maxWindDir, maxWindGust, minWindChill, maxHeatIndex, maxSolarRadAvg FROM DHL USE INDEX (PRIMARY) WHERE YearMonthDay = INSymd && vpID = INSvpID;

SELECT sum(t1.ET) INTO NEWsumEvpT FROM Hourly AS t1 INNER JOIN DT AS t2 USE INDEX (YMDvpID) USING (RecID) WHERE t2.YearMonthDay = INSymd && vpID = INSvpID;

IF NewDewPoint IS NOT NULL && (NewDewPoint > maxDewPoint || maxDewPoint IS NULL) THEN
    UPDATE DHL SET HourlyDewPointHigh = NewDewPoint, HourlyDewPointHighDT = INSdatetime, F = NewF WHERE YearMonthDay = INSymd && vpID = INSvpID;
END IF;

IF NewDewPoint IS NOT NULL && (NewDewPoint < minDewPoint || minDewPoint IS NULL) THEN
    UPDATE DHL SET HourlyDewPointLow = NewDewPoint, HourlyDewPointLowDT = INSdatetime, F = NewF WHERE YearMonthDay = INSymd && vpID = INSvpID;
END IF;

IF NewTHSWIndex IS NOT NULL && (NewTHSWIndex > maxTHSWIndex || maxTHSWIndex IS NULL) THEN
    UPDATE DHL SET HourlyTHSWIndex = NewTHSWIndex, HourlyTHSWIndexDT = INSdatetime, F = NewF WHERE YearMonthDay = INSymd && vpID = INSvpID;
END IF;

IF NewEvpT IS NOT NULL && (NewEvpT > maxEvpT || maxEvpT IS NULL) THEN
    UPDATE DHL SET HourlyETHigh = NewEvpT, HourlyETHighDT = INSdatetime, F = NewF WHERE YearMonthDay = INSymd && vpID = INSvpID;
END IF;

IF NEWsumEvpT IS NOT NULL && (NEWsumEvpT > sumEvpT || sumEvpT IS NULL) THEN
    UPDATE DHL SET HourlyTotalET = NEWsumEvpT, F = NewF WHERE YearMonthDay = INSymd && vpID = INSvpID;
END IF;

IF NewOutTemp IS NOT NULL && (NewOutTemp > maxOutTemp || maxOutTemp IS NULL) THEN
    UPDATE DHL SET HourlyOutTempHigh = NewOutTemp, HourlyOutTempHighDT = INSdatetime, F = NewF WHERE YearMonthDay = INSymd && vpID = INSvpID;
END IF;

IF NewOutTemp IS NOT NULL && (NewOutTemp < minOutTemp || minOutTemp IS NULL) THEN
    UPDATE DHL SET HourlyOutTempLow = NewOutTemp, HourlyOutTempLowDT = INSdatetime, F = NewF WHERE YearMonthDay = INSymd && vpID = INSvpID;
END IF;

IF NewInTemp IS NOT NULL && (NewInTemp > maxInTemp || maxInTemp IS NULL) THEN
    UPDATE DHL SET HourlyInTempHigh = NewInTemp, HourlyInTempHighDT = INSdatetime, F = NewF WHERE YearMonthDay = INSymd && vpID = INSvpID;
END IF;

IF NewInTemp IS NOT NULL && (NewInTemp < minInTemp || minInTemp IS NULL) THEN
    UPDATE DHL SET HourlyInTempLow = NewInTemp, HourlyInTempLowDT = INSdatetime, F = NewF WHERE YearMonthDay = INSymd && vpID = INSvpID;
END IF;

IF NewOutHum IS NOT NULL && (NewOutHum > maxOutHum || maxOutHum IS NULL) THEN
    UPDATE DHL SET HourlyOutHumHigh = NewOutHum, HourlyOutHumHighDT = INSdatetime, F = NewF WHERE YearMonthDay = INSymd && vpID = INSvpID;
END IF;

IF NewOutHum IS NOT NULL && (NewOutHum < minOutHum || minOutHum IS NULL) THEN
    UPDATE DHL SET HourlyOutHumLow = NewOutHum, HourlyOutHumLowDT = INSdatetime, F = NewF WHERE YearMonthDay = INSymd && vpID = INSvpID;
END IF;

IF NewInHum IS NOT NULL && (NewInHum > maxInHum || maxInHum IS NULL) THEN
    UPDATE DHL SET HourlyInHumHigh = NewInHum, HourlyInHumHighDT = INSdatetime, F = NewF WHERE YearMonthDay = INSymd && vpID = INSvpID;
END IF;

IF NewInHum IS NOT NULL && (NewInHum < minInHum || minInHum IS NULL) THEN
    UPDATE DHL SET HourlyInHumLow = NewInHum, HourlyInHumLowDT = INSdatetime, F = NewF WHERE YearMonthDay = INSymd && vpID = INSvpID;
END IF;

IF NewBarometer IS NOT NULL && (NewBarometer > maxBaro || maxBaro IS NULL) THEN
    UPDATE DHL SET HourlyBarometerHigh = NewBarometer, HourlyBarometerHighDT = INSdatetime, F = NewF WHERE YearMonthDay = INSymd && vpID = INSvpID;
END IF;

IF NewBarometer IS NOT NULL && (NewBarometer < minBaro || minBaro IS NULL) THEN
    UPDATE DHL SET HourlyBarometerLow = NewBarometer, HourlyBarometerLowDT = INSdatetime, F = NewF WHERE YearMonthDay = INSymd && vpID = INSvpID;
END IF;

IF NewRainAmount IS NOT NULL && (NewRainAmount > maxRain || maxRain IS NULL) THEN
    UPDATE DHL SET HourlyRainAmount = NewRainAmount, HourlyRainAmountDT = INSdatetime, F = NewF WHERE YearMonthDay = INSymd && vpID = INSvpID;
END IF;

IF NewRainRate IS NOT NULL && (NewRainRate > maxRainRate || maxRainRate IS NULL) THEN
    UPDATE DHL SET HourlyRainRate = NewRainRate, HourlyRainRateDT = INSdatetime, F = NewF WHERE YearMonthDay = INSymd && vpID = INSvpID;
END IF;

IF NewWindSpeed IS NOT NULL && (NewWindSpeed > maxWindSpeed || maxWindSpeed IS NULL) THEN
    UPDATE DHL SET HourlyWindSpeedAvg = NewWindSpeed, HourlyWindSpeedAvgDT = INSdatetime, F = NewF WHERE YearMonthDay = INSymd && vpID = INSvpID;
END IF;

IF NewWindDir IS NOT NULL && (NewWindDir > maxWindDir || maxWindDir IS NULL) THEN
    UPDATE DHL SET HourlyWindDirAvg = NewWindDir, HourlyWindDirAvgDT = INSdatetime, F = NewF WHERE YearMonthDay = INSymd && vpID = INSvpID;
END IF;

IF NewWindGust IS NOT NULL && (NewWindGust > maxWindGust || maxWindGust IS NULL) THEN
    UPDATE DHL SET HourlyWindSpeedHigh = NewWindGust, HourlyWindSpeedHighDT = INSdatetime, F = NewF WHERE YearMonthDay = INSymd && vpID = INSvpID;
END IF;

IF NewWindChill IS NOT NULL && (NewWindChill < minWindChill || minWindChill IS NULL) THEN
    UPDATE DHL SET HourlyWindChill = NewWindChill, HourlyWindChillDT = INSdatetime, F = NewF WHERE YearMonthDay = INSymd && vpID = INSvpID;
END IF;

IF NewHeatIndex IS NOT NULL && (NewHeatIndex > maxHeatIndex || maxHeatIndex IS NULL) THEN
    UPDATE DHL SET HourlyHeatIndex = NewHeatIndex, HourlyHeatIndexDT = INSdatetime, F = NewF WHERE YearMonthDay = INSymd && vpID = INSvpID;
END IF;

IF NewSolarRadAvg IS NOT NULL && (NewSolarRadAvg > maxSolarRadAvg || maxSolarRadAvg IS NULL) THEN
    UPDATE DHL SET HourlySolarRad = NewSolarRadAvg, HourlySolarRadDT = INSdatetime, F = NewF WHERE YearMonthDay = INSymd && vpID = INSvpID;
END IF;

SELECT HourlyDewPointHigh, HourlyDewPointLow, HourlyTHSWIndex, HourlyETHigh, HourlyTotalET INTO maxDewPoint, minDewPoint, maxTHSWIndex, maxEvpT, sumEvpT FROM MHL USE INDEX (PRIMARY) WHERE YearMonth = INSym && vpID = INSvpID;

SELECT sum(HourlyTotalET) INTO NEWsumEvpT FROM DHL USE INDEX (YMvpID) WHERE YearMonth = INSym && vpID = INSvpID;

IF NewDewPoint IS NOT NULL && (NewDewPoint > maxDewPoint || maxDewPoint IS NULL) THEN
    UPDATE MHL SET HourlyDewPointHigh = NewDewPoint, HourlyDewPointHighDT = INSdatetime, F = NewF WHERE YearMonth = INSym && vpID = INSvpID;
END IF;

IF NewDewPoint IS NOT NULL && (NewDewPoint < minDewPoint || minDewPoint IS NULL) THEN
    UPDATE MHL SET HourlyDewPointLow = NewDewPoint, HourlyDewPointLowDT = INSdatetime, F = NewF WHERE YearMonth = INSym && vpID = INSvpID;
END IF;

IF NewTHSWIndex IS NOT NULL && (NewTHSWIndex > maxTHSWIndex || maxTHSWIndex IS NULL) THEN
    UPDATE MHL SET HourlyTHSWIndex = NewTHSWIndex, HourlyTHSWIndexDT = INSdatetime, F = NewF WHERE YearMonth = INSym && vpID = INSvpID;
END IF;

IF NewEvpT IS NOT NULL && (NewEvpT > maxEvpT || maxEvpT IS NULL) THEN
    UPDATE MHL SET HourlyETHigh = NewEvpT, HourlyETHighDT = INSdatetime, F = NewF WHERE YearMonth = INSym && vpID = INSvpID;
END IF;

IF NEWsumEvpT IS NOT NULL && (NEWsumEvpT > sumEvpT || sumEvpT IS NULL) THEN
    UPDATE MHL SET HourlyTotalET = NEWsumEvpT, F = NewF WHERE YearMonth = INSym && vpID = INSvpID;
END IF;

SELECT HourlyDewPointHigh, HourlyDewPointLow, HourlyTHSWIndex, HourlyETHigh, HourlyTotalET INTO maxDewPoint, minDewPoint, maxTHSWIndex, maxEvpT, sumEvpT FROM YHL USE INDEX (PRIMARY) WHERE bYear = INSy && vpID = INSvpID;

SELECT sum(HourlyTotalET) INTO NEWsumEvpT FROM MHL USE INDEX (PRIMARY) WHERE bYear = INSy && vpID = INSvpID;

IF NewDewPoint IS NOT NULL && (NewDewPoint > maxDewPoint || maxDewPoint IS NULL) THEN
    UPDATE YHL SET HourlyDewPointHigh = NewDewPoint, HourlyDewPointHighDT = INSdatetime, F = NewF WHERE bYear = INSy && vpID = INSvpID;
END IF;

IF NewDewPoint IS NOT NULL && (NewDewPoint < minDewPoint || minDewPoint IS NULL) THEN
    UPDATE YHL SET HourlyDewPointLow = NewDewPoint, HourlyDewPointLowDT = INSdatetime, F = NewF WHERE bYear = INSy && vpID = INSvpID;
END IF;

IF NewTHSWIndex IS NOT NULL && (NewTHSWIndex > maxTHSWIndex || maxTHSWIndex IS NULL) THEN
    UPDATE YHL SET HourlyTHSWIndex = NewTHSWIndex, HourlyTHSWIndexDT = INSdatetime, F = NewF WHERE bYear = INSy && vpID = INSvpID;
END IF;

IF NewEvpT IS NOT NULL && (NewEvpT > maxEvpT || maxEvpT IS NULL) THEN
    UPDATE YHL SET HourlyETHigh = NewEvpT, HourlyETHighDT = INSdatetime, F = NewF WHERE bYear = INSy && vpID = INSvpID;
END IF;

IF NEWsumEvpT IS NOT NULL && (NEWsumEvpT > sumEvpT || sumEvpT IS NULL) THEN
    UPDATE YHL SET HourlyTotalET = NEWsumEvpT, F = NewF WHERE bYear = INSy && vpID = INSvpID;
END IF;

END$$

CREATE PROCEDURE Hourly_HL_SET(IN nRecID BIGINT)
MODIFIES SQL DATA
BEGIN

DECLARE INSdate DATE;
DECLARE INSymd INTEGER(8);
DECLARE INSym INTEGER(6);
DECLARE INSy SMALLINT(4);
DECLARE INSm SMALLINT(2);
DECLARE INSvpID TINYINT;

DECLARE maxDewPoint SMALLINT;
DECLARE maxDewPointDT DATETIME;
DECLARE minDewPoint SMALLINT;
DECLARE minDewPointDT DATETIME;

DECLARE maxTHSWIndex SMALLINT;
DECLARE maxTHSWIndexDT DATETIME;

DECLARE sumEvpT DECIMAL(6,3);
DECLARE maxEvpT DECIMAL(6,3);
DECLARE maxEvpTDT DATETIME;

DECLARE maxOutTemp SMALLINT;
DECLARE maxOutTempDT DATETIME;
DECLARE minOutTemp SMALLINT;
DECLARE minOutTempDT DATETIME;
DECLARE maxInTemp SMALLINT;
DECLARE maxInTempDT DATETIME;
DECLARE minInTemp SMALLINT;
DECLARE minInTempDT DATETIME;

DECLARE maxOutHum SMALLINT;
DECLARE maxOutHumDT DATETIME;
DECLARE minOutHum SMALLINT;
DECLARE minOutHumDT DATETIME;
DECLARE maxInHum SMALLINT;
DECLARE maxInHumDT DATETIME;
DECLARE minInHum SMALLINT;
DECLARE minInHumDT DATETIME;

DECLARE maxBaro DECIMAL(8,3);
DECLARE maxBaroDT DATETIME;
DECLARE minBaro DECIMAL(8,3);
DECLARE minBaroDT DATETIME;

DECLARE maxRain DECIMAL(6,2);
DECLARE maxRainDT DATETIME;
DECLARE maxRainRate DECIMAL(7,2);
DECLARE maxRainRateDT DATETIME;

DECLARE maxWindSpeed SMALLINT;
DECLARE maxWindSpeedDT DATETIME;
DECLARE maxWindDir SMALLINT;
DECLARE maxWindDirDT DATETIME;
DECLARE maxWindGust SMALLINT;
DECLARE maxWindGustDT DATETIME;

DECLARE minWindChill SMALLINT;
DECLARE minWindChillDT DATETIME;
DECLARE maxHeatIndex SMALLINT;
DECLARE maxHeatIndexDT DATETIME;
DECLARE maxSolarRadAvg SMALLINT;
DECLARE maxSolarRadAvgDT DATETIME;

DECLARE curF CHAR(1);

SELECT bDate, YearMonthDay, YearMonth, bYear, bMonth, vpID INTO INSdate, INSymd, INSym, INSy, INSm, INSvpID FROM DT USE INDEX (RecID) WHERE RecID = nRecID;

SELECT F INTO curF FROM Hourly USE INDEX (PRIMARY) WHERE RecID = nRecID;

SELECT max(t1.DewPoint), min(t1.DewPoint), max(t1.THSWIndex), max(t1.ET), max(t1.OutTemp), min(t1.OutTemp), max(t1.InTemp), min(t1.InTemp), max(t1.OutHum), min(t1.OutHum), max(t1.InHum), min(t1.InHum), max(t1.Barometer), min(t1.Barometer), max(t1.RainAmount), max(t1.RainRate), max(t1.WindSpeedAvg), max(t1.WindDirAvg), max(t1.WindSpeedHigh), max(t1.WindChill), max(t1.HeatIndex), max(t1.SolarRadAvg), sum(t1.ET) INTO maxDewPoint, minDewPoint, maxTHSWIndex, maxEvpT, maxOutTemp, minOutTemp, maxInTemp, minInTemp, maxOutHum, minOutHum, maxInHum, minInHum, maxBaro, minBaro, maxRain, maxRainRate, maxWindSpeed, maxWindDir, maxWindGust, minWindChill, maxHeatIndex, maxSolarRadAvg, sumEvpT FROM Hourly AS t1 INNER JOIN DT AS t2 USE INDEX (YMDvpID) USING (RecID) WHERE t2.YearMonthDay = INSymd && t2.vpID = INSvpID;

SELECT t2.bDateTime INTO maxDewPointDT FROM Hourly AS t1 INNER JOIN DT AS t2 USE INDEX (YMDvpID) USING (RecID) WHERE t1.DewPoint = maxDewPoint && t2.YearMonthDay = INSymd && t2.vpID = INSvpID ORDER BY t2.bDateTime LIMIT 1;

SELECT t2.bDateTime INTO minDewPointDT FROM Hourly AS t1 INNER JOIN DT AS t2 USE INDEX (YMDvpID) USING (RecID) WHERE t1.DewPoint = minDewPoint && t2.YearMonthDay = INSymd && t2.vpID = INSvpID ORDER BY t2.bDateTime LIMIT 1;

SELECT t2.bDateTime INTO maxTHSWIndexDT FROM Hourly AS t1 INNER JOIN DT AS t2 USE INDEX (YMDvpID) USING (RecID) WHERE t1.THSWIndex = maxTHSWIndex && t2.YearMonthDay = INSymd && t2.vpID = INSvpID ORDER BY t2.bDateTime LIMIT 1;

SELECT t2.bDateTime INTO maxEvpTDT FROM Hourly AS t1 INNER JOIN DT AS t2 USE INDEX (YMDvpID) USING (RecID) WHERE t1.ET = maxEvpT && t2.YearMonthDay = INSymd && t2.vpID = INSvpID ORDER BY t2.bDateTime LIMIT 1;

SELECT t2.bDateTime INTO maxOutTempDT FROM Hourly AS t1 INNER JOIN DT AS t2 USE INDEX (YMDvpID) USING (RecID) WHERE t1.OutTemp = maxOutTemp && t2.YearMonthDay = INSymd && t2.vpID = INSvpID ORDER BY t2.bDateTime LIMIT 1;

SELECT t2.bDateTime INTO minOutTempDT FROM Hourly AS t1 INNER JOIN DT AS t2 USE INDEX (YMDvpID) USING (RecID) WHERE t1.OutTemp = minOutTemp && t2.YearMonthDay = INSymd && t2.vpID = INSvpID ORDER BY t2.bDateTime LIMIT 1;

SELECT t2.bDateTime INTO maxInTempDT FROM Hourly AS t1 INNER JOIN DT AS t2 USE INDEX (YMDvpID) USING (RecID) WHERE t1.InTemp = maxInTemp && t2.YearMonthDay = INSymd && t2.vpID = INSvpID ORDER BY t2.bDateTime LIMIT 1;

SELECT t2.bDateTime INTO minInTempDT FROM Hourly AS t1 INNER JOIN DT AS t2 USE INDEX (YMDvpID) USING (RecID) WHERE t1.InTemp = minInTemp && t2.YearMonthDay = INSymd && t2.vpID = INSvpID ORDER BY t2.bDateTime LIMIT 1;

SELECT t2.bDateTime INTO maxOutHumDT FROM Hourly AS t1 INNER JOIN DT AS t2 USE INDEX (YMDvpID) USING (RecID) WHERE t1.OutHum = maxOutHum && t2.YearMonthDay = INSymd && t2.vpID = INSvpID ORDER BY t2.bDateTime LIMIT 1;

SELECT t2.bDateTime INTO minOutHumDT FROM Hourly AS t1 INNER JOIN DT AS t2 USE INDEX (YMDvpID) USING (RecID) WHERE t1.OutHum = minOutHum && t2.YearMonthDay = INSymd && t2.vpID = INSvpID ORDER BY t2.bDateTime LIMIT 1;

SELECT t2.bDateTime INTO maxInHumDT FROM Hourly AS t1 INNER JOIN DT AS t2 USE INDEX (YMDvpID) USING (RecID) WHERE t1.InHum = maxInHum && t2.YearMonthDay = INSymd && t2.vpID = INSvpID ORDER BY t2.bDateTime LIMIT 1;

SELECT t2.bDateTime INTO minInHumDT FROM Hourly AS t1 INNER JOIN DT AS t2 USE INDEX (YMDvpID) USING (RecID) WHERE t1.InHum = minInHum && t2.YearMonthDay = INSymd && t2.vpID = INSvpID ORDER BY t2.bDateTime LIMIT 1;

SELECT t2.bDateTime INTO maxBaroDT FROM Hourly AS t1 INNER JOIN DT AS t2 USE INDEX (YMDvpID) USING (RecID) WHERE t1.Barometer = maxBaro && t2.YearMonthDay = INSymd && t2.vpID = INSvpID ORDER BY t2.bDateTime LIMIT 1;

SELECT t2.bDateTime INTO minBaroDT FROM Hourly AS t1 INNER JOIN DT AS t2 USE INDEX (YMDvpID) USING (RecID) WHERE t1.Barometer = minBaro && t2.YearMonthDay = INSymd && t2.vpID = INSvpID ORDER BY t2.bDateTime LIMIT 1;

SELECT t2.bDateTime INTO maxRainDT FROM Hourly AS t1 INNER JOIN DT AS t2 USE INDEX (YMDvpID) USING (RecID) WHERE t1.RainAmount = maxRain && t2.YearMonthDay = INSymd && t2.vpID = INSvpID ORDER BY t2.bDateTime LIMIT 1;

SELECT t2.bDateTime INTO maxRainRateDT FROM Hourly AS t1 INNER JOIN DT AS t2 USE INDEX (YMDvpID) USING (RecID) WHERE t1.RainRate = maxRainRate && t2.YearMonthDay = INSymd && t2.vpID = INSvpID ORDER BY t2.bDateTime LIMIT 1;

SELECT t2.bDateTime INTO maxWindSpeedDT FROM Hourly AS t1 INNER JOIN DT AS t2 USE INDEX (YMDvpID) USING (RecID) WHERE t1.WindSpeedAvg = maxWindSpeed && t2.YearMonthDay = INSymd && t2.vpID = INSvpID ORDER BY t2.bDateTime LIMIT 1;

SELECT t2.bDateTime INTO maxWindDirDT FROM Hourly AS t1 INNER JOIN DT AS t2 USE INDEX (YMDvpID) USING (RecID) WHERE t1.WindDirAvg = maxWindDir && t2.YearMonthDay = INSymd && t2.vpID = INSvpID ORDER BY t2.bDateTime LIMIT 1;

SELECT t2.bDateTime INTO maxWindGustDT FROM Hourly AS t1 INNER JOIN DT AS t2 USE INDEX (YMDvpID) USING (RecID) WHERE t1.WindSpeedHigh = maxWindGust && t2.YearMonthDay = INSymd && t2.vpID = INSvpID ORDER BY t2.bDateTime LIMIT 1;

SELECT t2.bDateTime INTO minWindChillDT FROM Hourly AS t1 INNER JOIN DT AS t2 USE INDEX (YMDvpID) USING (RecID) WHERE t1.WindChill = minWindChill && t2.YearMonthDay = INSymd && t2.vpID = INSvpID ORDER BY t2.bDateTime LIMIT 1;

SELECT t2.bDateTime INTO maxHeatIndexDT FROM Hourly AS t1 INNER JOIN DT AS t2 USE INDEX (YMDvpID) USING (RecID) WHERE t1.HeatIndex = maxHeatIndex && t2.YearMonthDay = INSymd && t2.vpID = INSvpID ORDER BY t2.bDateTime LIMIT 1;

SELECT t2.bDateTime INTO maxSolarRadAvgDT FROM Hourly AS t1 INNER JOIN DT AS t2 USE INDEX (YMDvpID) USING (RecID) WHERE t1.SolarRadAvg = maxSolarRadAvg && t2.YearMonthDay = INSymd && t2.vpID = INSvpID ORDER BY t2.bDateTime LIMIT 1;

UPDATE DHL SET HourlyDewPointHigh = maxDewPoint, HourlyDewPointHighDT = maxDewPointDT, HourlyDewPointLow = minDewPoint, HourlyDewPointLowDT = minDewPointDT, HourlyTHSWIndex = maxTHSWIndex, HourlyTHSWIndexDT = maxTHSWIndexDT, HourlyTotalET = sumEvpT, HourlyETHigh = maxEvpT, HourlyETHighDT = maxEvpTDT, HourlyOutTempHigh = maxOutTemp, HourlyOutTempHighDT = maxOutTempDT, HourlyOutTempLow = minOutTemp, HourlyOutTempLowDT = minOutTempDT, HourlyInTempHigh = maxInTemp, HourlyInTempHighDT = maxInTempDT, HourlyInTempLow = minInTemp, HourlyInTempLowDT = minInTempDT, HourlyOutHumHigh = maxOutHum, HourlyOutHumHighDT = maxOutHumDT, HourlyOutHumLow = minOutHum, HourlyOutHumLowDT = minOutHumDT, HourlyInHumHigh = maxInHum, HourlyInHumHighDT = maxInHumDT, HourlyInHumLow = minInHum, HourlyInHumLowDT = minInHumDT, HourlyBarometerHigh = maxBaro, HourlyBarometerHighDT = maxBaroDT, HourlyBarometerLow = minBaro, HourlyBarometerLowDT = minBaroDT, HourlyRainAmount = maxRain, HourlyRainAmountDT = maxRainDT, HourlyRainRate = maxRainRate, HourlyRainRateDT = maxRainRateDT, HourlyWindSpeedAvg = maxWindSpeed, HourlyWindSpeedAvgDT = maxWindSpeedDT, HourlyWindDirAvg = maxWindDir, HourlyWindDirAvgDT = maxWindDirDT, HourlyWindSpeedHigh = maxWindGust, HourlyWindSpeedHighDT = maxWindGustDT, HourlyWindChill = minWindChill, HourlyWindChillDT = minWindChillDT, HourlyHeatIndex = maxHeatIndex, HourlyHeatIndexDT = maxHeatIndexDT, HourlySolarRad = maxSolarRadAvg, HourlySolarRadDT = maxSolarRadAvgDT, F = curF WHERE YearMonthDay = INSymd && vpID = INSvpID;

SELECT max(HourlyDewPointHigh), min(HourlyDewPointLow), max(HourlyTHSWIndex), max(HourlyETHigh), sum(HourlyTotalET) INTO maxDewPoint, minDewPoint, maxTHSWIndex, maxEvpT, sumEvpT FROM DHL USE INDEX (PRIMARY) WHERE YearMonth = INSym && vpID = INSvpID;

SELECT HourlyDewPointHighDT INTO maxDewPointDT FROM DHL USE INDEX (YMvpID) WHERE HourlyDewPointHigh = maxDewPoint && YearMonth = INSym && vpID = INSvpID LIMIT 1;

SELECT HourlyDewPointLowDT INTO minDewPointDT FROM DHL USE INDEX (YMvpID) WHERE HourlyDewPointLow = minDewPoint && YearMonth = INSym && vpID = INSvpID LIMIT 1;

SELECT HourlyTHSWIndexDT INTO maxTHSWIndexDT FROM DHL USE INDEX (YMvpID) WHERE HourlyTHSWIndex = maxTHSWIndex && YearMonth = INSym && vpID = INSvpID LIMIT 1;

SELECT HourlyETHighDT INTO maxEvpTDT FROM DHL USE INDEX (YMvpID) WHERE HourlyETHigh = maxEvpT && YearMonth = INSym && vpID = INSvpID LIMIT 1;

UPDATE MHL SET HourlyDewPointHigh = maxDewPoint, HourlyDewPointHighDT = maxDewPointDT, HourlyDewPointLow = minDewPoint, HourlyDewPointLowDT = minDewPointDT, HourlyTHSWIndex = maxTHSWIndex, HourlyTHSWIndexDT = maxTHSWIndexDT, HourlyTotalET = sumEvpT, HourlyETHigh = maxEvpT, HourlyETHighDT = maxEvpTDT, F = curF WHERE YearMonth = INSym && vpID = INSvpID;

SELECT max(HourlyDewPointHigh), min(HourlyDewPointLow), max(HourlyTHSWIndex), max(HourlyETHigh), sum(HourlyTotalET) INTO maxDewPoint, minDewPoint, maxTHSWIndex, maxEvpT, sumEvpT FROM MHL USE INDEX (YvpID) WHERE bYear = INSy && vpID = INSvpID;

SELECT HourlyDewPointHighDT INTO maxDewPointDT FROM MHL USE INDEX (YvpID) WHERE HourlyDewPointHigh = maxDewPoint && bYear = INSy && vpID = INSvpID LIMIT 1;

SELECT HourlyDewPointLowDT INTO minDewPointDT FROM MHL USE INDEX (YvpID) WHERE HourlyDewPointLow = minDewPoint && bYear = INSy && vpID = INSvpID LIMIT 1;

SELECT HourlyTHSWIndexDT INTO maxTHSWIndexDT FROM MHL USE INDEX (YvpID) WHERE HourlyTHSWIndex = maxTHSWIndex && bYear = INSy && vpID = INSvpID LIMIT 1;

SELECT HourlyETHighDT INTO maxEvpTDT FROM MHL USE INDEX (YvpID) WHERE HourlyETHigh = maxEvpT && bYear = INSy && vpID = INSvpID LIMIT 1;

UPDATE YHL SET HourlyDewPointHigh = maxDewPoint, HourlyDewPointHighDT = maxDewPointDT, HourlyDewPointLow = minDewPoint, HourlyDewPointLowDT = minDewPointDT, HourlyTHSWIndex = maxTHSWIndex, HourlyTHSWIndexDT = maxTHSWIndexDT, HourlyTotalET = sumEvpT, HourlyETHigh = maxEvpT, HourlyETHighDT = maxEvpTDT, F = curF WHERE bYear = INSy && vpID = INSvpID;

END$$

CREATE PROCEDURE Hourly_HL_RCL(IN nYMD INTEGER(8), nvpID TINYINT)
MODIFIES SQL DATA
BEGIN

DECLARE dRecID BIGINT;
DECLARE INSdate DATE;
DECLARE INSymd INTEGER(8);
DECLARE INSym INTEGER(6);
DECLARE INSy SMALLINT(4);
DECLARE INSm SMALLINT(2);
DECLARE INSd SMALLINT(2);
DECLARE dYMD INTEGER(8);

SELECT RecID, bDate, YearMonthDay, YearMonth, bYear, bMonth, bDay INTO dRecID, INSdate, INSymd, INSym, INSy, INSm, INSd FROM DT USE INDEX (YMDvpID) WHERE YearMonthDay = nYMD && vpID = nvpID LIMIT 1;

IF dRecID IS NOT NULL && dRecID > 0 THEN
    SELECT YearMonthDay INTO dYMD FROM YMD USE INDEX (PRIMARY) WHERE YearMonthDay = nYMD && vpID = nvpID LIMIT 1;
    IF dYMD IS NULL THEN
        INSERT IGNORE INTO YMD (YearMonthDay, YearMonth, bDate, bYear, bMonth, bDay, vpID) VALUES (INSymd, INSym, INSdate, INSy, INSm, INSd, nvpID);
    END IF;
    CALL Hourly_HL_SET(dRecID);
END IF;

END$$

CREATE TRIGGER Hourly_INS AFTER INSERT ON Hourly
FOR EACH ROW
BEGIN
CALL Hourly_HL_UPD(NEW.RecID, NEW.Barometer, NEW.DewPoint, NEW.ET, NEW.HeatIndex, NEW.InTemp, NEW.OutTemp, NEW.InHum, NEW.OutHum, NEW.RainAmount, NEW.RainRate, NEW.SolarRadAvg, NEW.THSWIndex, NEW.WindChill, NEW.WindDirAvg, NEW.WindSpeedHigh, NEW.WindSpeedAvg, NEW.F);
END$$

CREATE TRIGGER Hourly_UPD AFTER UPDATE ON Hourly
FOR EACH ROW
BEGIN
CALL Hourly_HL_UPD(NEW.RecID, NEW.Barometer, NEW.DewPoint, NEW.ET, NEW.HeatIndex, NEW.InTemp, NEW.OutTemp, NEW.InHum, NEW.OutHum, NEW.RainAmount, NEW.RainRate, NEW.SolarRadAvg, NEW.THSWIndex, NEW.WindChill, NEW.WindDirAvg, NEW.WindSpeedHigh, NEW.WindSpeedAvg, NEW.F);
END$$

CREATE TRIGGER Hourly_DEL AFTER DELETE ON Hourly
FOR EACH ROW
BEGIN
CALL Hourly_HL_SET(OLD.RecID);
END$$

DELIMITER ;


