USE Weather

DELIMITER $$

CREATE PROCEDURE Rain_HL_UPD(IN nRecID BIGINT, NewDayAmount decimal(6,2), NewRainRate decimal(7,2), NewET decimal(7,3), NewF CHAR(1))
MODIFIES SQL DATA
BEGIN

DECLARE INSdatetime DATETIME;
DECLARE INSdate DATE;
DECLARE INSymd INTEGER(8);
DECLARE INSym INTEGER(6);
DECLARE INSy SMALLINT(4);
DECLARE INSm SMALLINT(2);
DECLARE INSvpID TINYINT;

DECLARE curTotalRain decimal(6,2);
DECLARE maxRainRate decimal(7,2);
DECLARE sumTotalRain decimal(6,2);
DECLARE dayRainHigh decimal(6,2);
DECLARE curTotalET decimal(7,3);
DECLARE curETDayTotal decimal(7,3);
DECLARE sumTotalET decimal(7,3);

SELECT bDateTime, bDate, YearMonthDay, YearMonth, bYear, bMonth, vpID INTO INSdatetime, INSdate, INSymd, INSym, INSy, INSm, INSvpID FROM DT USE INDEX (RecID) WHERE RecID = nRecID;

SELECT TotalRain, RainRateHigh, TotalET, ETDayTotal INTO curTotalRain, maxRainRate, curTotalET, curETDayTotal FROM DHL USE INDEX (PRIMARY) WHERE YearMonthDay = INSymd && vpID = INSvpID;

IF NewDayAmount IS NOT NULL && (NewDayAmount > curTotalRain || curTotalRain IS NULL) THEN
    UPDATE DHL SET TotalRain = NewDayAmount, RainDayTotal = NewDayAmount, RainDayTotalDT = INSdatetime, F = NewF WHERE YearMonthDay = INSymd && vpID = INSvpID;
    SELECT sum(TotalRain) INTO sumTotalRain FROM DHL USE INDEX (YMvpID) WHERE YearMonth = INSym && vpID = INSvpID;
END IF;

IF NewET IS NOT NULL && (NewET > curTotalET || curTotalET IS NULL || NewET > curETDayTotal || curETDayTotal IS NULL) THEN
    UPDATE DHL SET TotalET = NewET, ETDayTotal = NewET, ETDayTotalDT = INSdatetime, F = NewF WHERE YearMonthDay = INSymd && vpID = INSvpID;
    SELECT sum(TotalET) INTO sumTotalET FROM DHL USE INDEX (YMvpID) WHERE YearMonth = INSym && vpID = INSvpID;
END IF;

IF NewRainRate IS NOT NULL && (NewRainRate > maxRainRate || maxRainRate IS NULL) THEN
    UPDATE DHL SET RainRateHigh = NewRainRate, RainRateHighDT = INSdatetime, F = NewF WHERE YearMonthDay = INSymd && vpID = INSvpID;
END IF;

SELECT TotalRain, RainRateHigh, RainDayTotal, TotalET, ETDayTotal INTO curTotalRain, maxRainRate, dayRainHigh, curTotalET, curETDayTotal FROM MHL USE INDEX (PRIMARY) WHERE YearMonth = INSym && vpID = INSvpID;

IF NewDayAmount IS NOT NULL && (NewDayAmount > dayRainHigh || dayRainHigh IS NULL) THEN
    UPDATE MHL SET RainDayTotal = NewDayAmount, RainDayTotalDT = INSdatetime, F = NewF WHERE YearMonth = INSym && vpID = INSvpID;
END IF;

IF NewET IS NOT NULL && (NewET > curETDayTotal || curETDayTotal IS NULL) THEN
    UPDATE MHL SET ETDayTotal = NewET, ETDayTotalDT = INSdatetime, F = NewF WHERE YearMonth = INSym && vpID = INSvpID;
END IF;

IF sumTotalET IS NOT NULL && (sumTotalET > curTotalET || curTotalET IS NULL) THEN
    UPDATE MHL SET TotalET = sumTotalET, F = NewF WHERE YearMonth = INSym && vpID = INSvpID;
    SELECT sum(TotalET) INTO sumTotalET FROM MHL USE INDEX (YvpID) WHERE bYear = INSy && vpID = INSvpID;
END IF;

IF sumTotalRain IS NOT NULL && (sumTotalRain > curTotalRain || curTotalRain IS NULL) THEN
    UPDATE MHL SET TotalRain = sumTotalRain, F = NewF WHERE YearMonth = INSym && vpID = INSvpID;
    SELECT sum(TotalRain) INTO sumTotalRain FROM MHL USE INDEX (YvpID) WHERE bYear = INSy && vpID = INSvpID;
END IF;

IF NewRainRate IS NOT NULL && (NewRainRate > maxRainRate || maxRainRate IS NULL) THEN
    UPDATE MHL SET RainRateHigh = NewRainRate, RainRateHighDT = INSdatetime, F = NewF WHERE YearMonth = INSym && vpID = INSvpID;
END IF;

SELECT TotalRain, RainRateHigh, RainDayTotal, TotalET, ETDayTotal INTO curTotalRain, maxRainRate, dayRainHigh, curTotalET, curETDayTotal FROM YHL USE INDEX (PRIMARY) WHERE bYear = INSy && vpID = INSvpID;

IF NewDayAmount IS NOT NULL && (NewDayAmount > dayRainHigh || dayRainHigh IS NULL) THEN
    UPDATE YHL SET RainDayTotal = NewDayAmount, RainDayTotalDT = INSdatetime, F = NewF WHERE bYear = INSy && vpID = INSvpID;
END IF;

IF NewET IS NOT NULL && (NewET > curETDayTotal || curETDayTotal IS NULL) THEN
    UPDATE YHL SET ETDayTotal = NewET, ETDayTotalDT = INSdatetime, F = NewF WHERE bYear = INSy && vpID = INSvpID;
END IF;

IF sumTotalET IS NOT NULL && (sumTotalET > curTotalET || curTotalET IS NULL) THEN
    UPDATE YHL SET TotalET = sumTotalET, F = NewF WHERE bYear = INSy && vpID = INSvpID;
END IF;

IF sumTotalRain IS NOT NULL && (sumTotalRain > curTotalRain || curTotalRain IS NULL) THEN
    UPDATE YHL SET TotalRain = sumTotalRain, F = NewF WHERE bYear = INSy && vpID = INSvpID;
END IF;

IF NewRainRate IS NOT NULL && (NewRainRate > maxRainRate || maxRainRate IS NULL) THEN
    UPDATE YHL SET RainRateHigh = NewRainRate, RainRateHighDT = INSdatetime, F = NewF WHERE bYear = INSy && vpID = INSvpID;
END IF;

END$$

CREATE PROCEDURE Rain_HL_SET(IN nRecID BIGINT)
MODIFIES SQL DATA
BEGIN

DECLARE INSdate DATE;
DECLARE INSymd INTEGER(8);
DECLARE INSym INTEGER(6);
DECLARE INSy SMALLINT(4);
DECLARE INSm SMALLINT(2);
DECLARE INSvpID TINYINT;

DECLARE totDayAmount DECIMAL(6,2);
DECLARE totDayAmountDT DATETIME;
DECLARE sumTotalRain DECIMAL(6,2);

DECLARE maxRate DECIMAL(7,2);
DECLARE maxRateDT DATETIME;

DECLARE totET DECIMAL(7,3);
DECLARE sumET DECIMAL(7,3);
DECLARE totETDT DATETIME;

DECLARE curF CHAR(1);

SELECT bDate, YearMonthDay, YearMonth, bYear, bMonth, vpID INTO INSdate, INSymd, INSym, INSy, INSm, INSvpID FROM DT USE INDEX (RecID) WHERE RecID = nRecID;

SELECT F INTO curF FROM Rain USE INDEX (PRIMARY) WHERE RecID = nRecID;

SELECT max(t1.DayAmount), max(t1.Rate), max(t1.ET) INTO totDayAmount, maxRate, totET FROM Rain AS t1 INNER JOIN DT AS t2 USE INDEX (YMDvpID) USING (RecID) WHERE t2.YearMonthDay = INSymd && t2.vpID = INSvpID;
SELECT t2.bDateTime INTO totDayAmountDT FROM Rain AS t1 INNER JOIN DT AS t2 USE INDEX (YMDvpID) USING (RecID) WHERE DayAmount = totDayAmount && t2.YearMonthDay = INSymd && t2.vpID = INSvpID ORDER BY t2.bDateTime LIMIT 1;
SELECT t2.bDateTime INTO maxRateDT FROM Rain AS t1 INNER JOIN DT AS t2 USE INDEX (YMDvpID) USING (RecID) WHERE t1.Rate = maxRate && t2.YearMonthDay = INSymd && t2.vpID = INSvpID ORDER BY t2.bDateTime LIMIT 1;
SELECT t2.bDateTime INTO totETDT FROM Rain AS t1 INNER JOIN DT AS t2 USE INDEX (YMDvpID) USING (RecID) WHERE t1.ET = totET && t2.YearMonthDay = INSymd && t2.vpID = INSvpID ORDER BY t2.bDateTime LIMIT 1;

UPDATE DHL SET TotalRain = totDayAmount, RainDayTotal = totDayAmount, RainDayTotalDT = totDayAmountDT, RainRateHigh = maxRate, RainRateHighDT = maxRateDT, TotalET = totET, ETDayTotal = totET, ETDayTotalDT = totETDT, F = curF WHERE YearMonthDay = INSymd && vpID = INSvpID;

SELECT max(RainDayTotal), max(RainRateHigh), max(ETDayTotal) INTO totDayAmount, maxRate, totET FROM DHL USE INDEX (YMvpID) WHERE YearMonth = INSym && vpID = INSvpID;
SELECT sum(RainDayTotal) INTO sumTotalRain FROM DHL USE INDEX (YMvpID) WHERE YearMonth = INSym && vpID = INSvpID;
SELECT sum(ETDayTotal) INTO sumET FROM DHL USE INDEX (YMvpID) WHERE YearMonth = INSym && vpID = INSvpID;
SELECT RainDayTotalDT INTO totDayAmountDT FROM DHL USE INDEX (YMvpID) WHERE RainDayTotal = totDayAmount && YearMonth = INSym && vpID = INSvpID LIMIT 1;
SELECT RainRateHighDT INTO maxRateDT FROM DHL USE INDEX (YMvpID) WHERE RainRateHigh = maxRate && YearMonth = INSym && vpID = INSvpID LIMIT 1;
SELECT ETDayTotalDT INTO totETDT FROM DHL USE INDEX (YMvpID) WHERE ETDayTotal = totET && YearMonth = INSym && vpID = INSvpID LIMIT 1;

UPDATE MHL SET TotalRain = sumTotalRain, RainDayTotal = totDayAmount, RainDayTotalDT = RainDayTotalDT, RainRateHigh = maxRate, RainRateHighDT = maxRateDT, TotalET = sumET, ETDayTotal = totET, ETDayTotalDT = totETDT, F = curF WHERE YearMonth = INSym && vpID = INSvpID;

SELECT max(RainDayTotal), max(RainRateHigh) INTO totDayAmount, maxRate FROM MHL USE INDEX (YvpID) WHERE bYear = INSy && vpID = INSvpID;
SELECT sum(TotalRain) INTO sumTotalRain FROM MHL USE INDEX (YvpID) WHERE bYear = INSy && vpID = INSvpID;
SELECT sum(TotalET) INTO sumET FROM MHL USE INDEX (YvpID) WHERE bYear = INSy && vpID = INSvpID;
SELECT RainDayTotalDT INTO totDayAmountDT FROM MHL USE INDEX (YvpID) WHERE RainDayTotal = totDayAmount && bYear = INSy && vpID = INSvpID LIMIT 1;
SELECT RainRateHighDT INTO maxRateDT FROM MHL USE INDEX (YvpID) WHERE RainRateHigh = maxRate && bYear = INSy && vpID = INSvpID LIMIT 1;
SELECT ETDayTotalDT INTO totETDT FROM MHL USE INDEX (YvpID) WHERE ETDayTotal = totET && bYear = INSy && vpID = INSvpID LIMIT 1;

UPDATE YHL SET TotalRain = sumTotalRain, RainDayTotal = totDayAmount, RainDayTotalDT = RainDayTotalDT, RainRateHigh = maxRate, RainRateHighDT = maxRateDT, TotalET = sumET, ETDayTotal = totET, ETDayTotalDT = totETDT, F = curF WHERE bYear = INSy && vpID = INSvpID;

END$$

CREATE PROCEDURE Rain_HL_RCL(IN nYMD INTEGER(8), nvpID TINYINT)
MODIFIES SQL DATA
BEGIN

DECLARE dRecID BIGINT;
DECLARE INSdate DATE;
DECLARE INSymd INTEGER(8);
DECLARE INSym INTEGER(6);
DECLARE INSy SMALLINT(4);
DECLARE INSm SMALLINT(2);
DECLARE INSd SMALLINT(2);
DECLARE dYMD INTEGER(8);

SELECT RecID, bDate, YearMonthDay, YearMonth, bYear, bMonth, bDay INTO dRecID, INSdate, INSymd, INSym, INSy, INSm, INSd FROM DT USE INDEX (YMDvpID) WHERE YearMonthDay = nYMD && vpID = nvpID LIMIT 1;

IF dRecID IS NOT NULL && dRecID > 0 THEN
    SELECT YearMonthDay INTO dYMD FROM YMD USE INDEX (PRIMARY) where YearMonthDay = nYMD && vpID = nvpID LIMIT 1;
    IF dYMD IS NULL THEN
        INSERT IGNORE INTO YMD (YearMonthDay, YearMonth, bDate, bYear, bMonth, bDay, vpID) VALUES (INSymd, INSym, INSdate, INSy, INSm, INSd, nvpID);
    END IF;
    CALL Rain_HL_SET(dRecID);
END IF;

END$$

CREATE TRIGGER Rain_INS AFTER INSERT ON Rain
FOR EACH ROW
BEGIN
CALL Rain_HL_UPD(NEW.RecID,NEW.DayAmount,NEW.Rate,NEW.ET,NEW.F);
END$$

CREATE TRIGGER Rain_UPD AFTER UPDATE ON Rain
FOR EACH ROW
BEGIN
CALL Rain_HL_UPD(NEW.RecID,NEW.DayAmount,NEW.Rate,NEW.ET,NEW.F);
END$$

CREATE TRIGGER Rain_DEL AFTER DELETE ON Rain
FOR EACH ROW
BEGIN
CALL Rain_HL_SET(OLD.RecID);
END$$


DELIMITER ;


