USE Weather

DELIMITER $$

CREATE PROCEDURE Solar_HL_UPD(IN nRecID BIGINT, NewSolarRad SMALLINT)
MODIFIES SQL DATA
BEGIN

DECLARE INSdatetime DATETIME;
DECLARE INSdate DATE;
DECLARE INSymd INTEGER(8);
DECLARE INSym INTEGER(6);
DECLARE INSy SMALLINT(4);
DECLARE INSm SMALLINT(2);
DECLARE INSvpID TINYINT;

DECLARE maxSolarRad SMALLINT;

SELECT bDateTime, bDate, YearMonthDay, YearMonth, bYear, bMonth, vpID INTO INSdatetime, INSdate, INSymd, INSym, INSy, INSm, INSvpID FROM DT USE INDEX (RecID) WHERE RecID = nRecID;

SELECT SolarRadHigh INTO maxSolarRad FROM DHL USE INDEX (PRIMARY) WHERE YearMonthDay = INSymd && vpID = INSvpID;

IF NewSolarRad IS NOT NULL && (NewSolarRad > maxSolarRad || maxSolarRad IS NULL) THEN
    UPDATE DHL SET SolarRadHigh = NewSolarRad, SolarRadHighDT = INSdatetime WHERE YearMonthDay = INSymd && vpID = INSvpID;
END IF;

SELECT SolarRadHigh INTO maxSolarRad FROM MHL USE INDEX (PRIMARY) WHERE YearMonth = INSym && vpID = INSvpID;

IF NewSolarRad IS NOT NULL && (NewSolarRad > maxSolarRad || maxSolarRad IS NULL) THEN
    UPDATE MHL SET SolarRadHigh = NewSolarRad, SolarRadHighDT = INSdatetime WHERE YearMonth = INSym && vpID = INSvpID;
END IF;

SELECT SolarRadHigh INTO maxSolarRad FROM YHL USE INDEX (PRIMARY) WHERE bYear = INSy && vpID = INSvpID;

IF NewSolarRad IS NOT NULL && (NewSolarRad > maxSolarRad || maxSolarRad IS NULL) THEN
    UPDATE YHL SET SolarRadHigh = NewSolarRad, SolarRadHighDT = INSdatetime WHERE bYear = INSy && vpID = INSvpID;
END IF;

END$$

CREATE PROCEDURE Solar_HL_SET(IN nRecID BIGINT)
MODIFIES SQL DATA
BEGIN

DECLARE INSdate DATE;
DECLARE INSymd INTEGER(8);
DECLARE INSym INTEGER(6);
DECLARE INSy SMALLINT(4);
DECLARE INSm SMALLINT(2);
DECLARE INSvpID TINYINT;

DECLARE maxSolarRad SMALLINT;
DECLARE maxSolarRadDT DATETIME;

SELECT bDate, YearMonthDay, YearMonth, bYear, bMonth, vpID INTO INSdate, INSymd, INSym, INSy, INSm, INSvpID FROM DT USE INDEX (RecID) WHERE RecID = nRecID;

SELECT max(t1.Radiation) INTO maxSolarRad FROM Solar AS t1 INNER JOIN DT AS t2 USE INDEX (YMDvpID) USING (RecID) WHERE t2.YearMonthDay = INSymd && t2.vpID = INSvpID;

SELECT t2.bDateTime INTO maxSolarRadDT FROM Solar AS t1 INNER JOIN DT AS t2 USE INDEX (YMDvpID) USING (RecID) WHERE t1.Radiation = maxSolarRad && t2.YearMonthDay = INSymd && t2.vpID = INSvpID ORDER BY t2.bDateTime LIMIT 1;

UPDATE DHL SET SolarRadHigh = maxSolarRad, SolarRadHighDT = maxSolarRadDT WHERE YearMonthDay = INSymd && vpID = INSvpID;

SELECT max(SolarRadHigh) INTO maxSolarRad FROM DHL USE INDEX (YMvpID) WHERE YearMonth = INSym && vpID = INSvpID;

SELECT SolarRadHighDT INTO maxSolarRadDT FROM DHL USE INDEX (YMvpID) WHERE SolarRadHigh = maxSolarRad && YearMonth = INSym && vpID = INSvpID LIMIT 1;

UPDATE MHL SET SolarRadHigh = maxSolarRad, SolarRadHighDT = maxSolarRadDT WHERE YearMonth = INSym && vpID = INSvpID;

SELECT max(SolarRadHigh) INTO maxSolarRad FROM MHL USE INDEX (YvpID) WHERE bYear = INSy && vpID = INSvpID;

SELECT SolarRadHighDT INTO maxSolarRadDT FROM MHL USE INDEX (YvpID) WHERE SolarRadHigh = maxSolarRad && bYear = INSy && vpID = INSvpID LIMIT 1;

UPDATE YHL SET SolarRadHigh = maxSolarRad, SolarRadHighDT = maxSolarRadDT WHERE bYear = INSy && vpID = INSvpID;

END$$

CREATE PROCEDURE Solar_HL_RCL(IN nYMD INTEGER(8), nvpID TINYINT)
MODIFIES SQL DATA
BEGIN

DECLARE dRecID BIGINT;
DECLARE INSdate DATE;
DECLARE INSymd INTEGER(8);
DECLARE INSym INTEGER(6);
DECLARE INSy SMALLINT(4);
DECLARE INSm SMALLINT(2);
DECLARE INSd SMALLINT(2);
DECLARE dYMD INTEGER(8);

SELECT RecID, bDate, YearMonthDay, YearMonth, bYear, bMonth, bDay INTO dRecID, INSdate, INSymd, INSym, INSy, INSm, INSd FROM DT USE INDEX (YMDvpID) WHERE YearMonthDay = nYMD && vpID = nvpId LIMIT 1;

IF dRecID IS NOT NULL && dRecID > 0 THEN
    SELECT YearMonthDay INTO dYMD FROM YMD USE INDEX (PRIMARY) WHERE YearMonthDay = nYMD && vpID = nvpId LIMIT 1;
    IF dYMD IS NULL THEN
        INSERT IGNORE INTO YMD (YearMonthDay, YearMonth, bDate, bYear, bMonth, bDay, vpID) VALUES (INSymd, INSym, INSdate, INSy, INSm, INSd, nvpID);
    END IF;
    CALL Solar_HL_SET(dRecID);
END IF;

END$$

CREATE TRIGGER Solar_INS AFTER INSERT ON Solar
FOR EACH ROW
BEGIN
CALL Solar_HL_UPD(NEW.RecID,NEW.RADIATION);
END$$

CREATE TRIGGER Solar_UPD AFTER UPDATE ON Solar
FOR EACH ROW
BEGIN
CALL Solar_HL_UPD(NEW.RecID,NEW.RADIATION);
END$$

CREATE TRIGGER Solar_DEL AFTER DELETE ON Solar
FOR EACH ROW
BEGIN
CALL Solar_HL_SET(OLD.RecID);
END$$


DELIMITER ;


