/**********************************************************************************/
/*  vproweather 1.0.9                                                             */
/*        A program for acquiring data from a Davis Vantage Pro Weather           */
/*        Station.                                                                */
/*                                                                                */
/*  Thanks to Paul Davis for his 'remserial' program, and Aaron Logue's           */
/*  'weastat' program.                                                            */
/*                                                                                */
/* (c)2004 Joe Jaworski    email: jj@joejaworski.com                              */
/* (c)2005-2013 Curt Blank email: curt@curtronics.com                             */
/* VPROWEATHER is covered under the GNU general license. See the attached         */
/* LICENSE file for details.                                                      */
/*                                                                                */
/* This software is provided as-is, without any expressed or implied              */
/* warranties, including, without limitation, the implied warranties of           */
/* merchantibility and fitness for any particular purpose.                        */
/*                                                                                */
/*     v0.8.0 cjb - add access to mySQL db                                        */
/*     v1.0.4  cjb - added -2 option for use with Vantage Pro 2 and removed       */
/*                   compile flag, now a runtime option                           */
/*                                                                                */
/* main.h.h - main header file                                                    */
/*                                                                                */
/**********************************************************************************/

#ifndef __MAIN_H__
#define __MAIN_H__

#ifdef __cplusplus
#define extern "C" {                /* respect c++ callers */
#endif

#define ProgramName     "vproweather"

/* imports */
extern void *getMemPtr(size_t size);
extern void ListRTDataFmt();

/* exports */

typedef int BOOL;                   /* yet another bool definition */
#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif

typedef unsigned char BYTE;
typedef unsigned short WORD;        
#ifndef bit64
typedef unsigned long DWORD;	
#endif
#ifdef bit64
typedef unsigned int DWORD;
#endif

#define LOBYTE(w)	((BYTE)(w))
#define HIBYTE(w)	((BYTE)(((WORD)(w) >> (BYTE)8) & 0xff))

#define ttyLCKloc       "/var/lock/LCK.."       /* location and prefix of serial port lock file */

int BAR_15_MIN;
int BAR_DAY_HIGHS;
int BAR_DAY_HIGH_TIMES;
int BAR_DAY_LOWS;
int BAR_DAY_LOW_TIMES;
int BAR_HOUR;
int BAR_MONTH_HIGHS;
int BAR_MONTH_LOWS;
int BAR_YEAR_HIGHS;
int BAR_YEAR_LOWS;
int BAUD_RATE;
int CHILL_DAY_LOWS;
int CHILL_DAY_LOW_TIMES;
int CHILL_HOUR;
int CHILL_MONTH_LOWS;
int CHILL_YEAR_LOWS;
int DEFAULT_RATE_GRAPH;
int DEW_DAY_HIGHS;
int DEW_DAY_HIGH_TIMES;
int DEW_DAY_LOWS;
int DEW_DAY_LOW_TIMES;
int DEW_HOUR;
int DEW_MONTH_HIGHS;
int DEW_MONTH_LOWS;
int DEW_YEAR_HIGHS;
int DEW_YEAR_LOWS;
int ET_DAY_TOTAL;
int ET_HOUR;
int ET_MONTH_TOTAL;
int ET_YEAR_TOTAL;
int HEAT_DAY_HIGHS;
int HEAT_DAY_HIGH_TIMES;
int HEAT_HOUR;
int HEAT_MONTH_HIGHS;
int HEAT_YEAR_HIGHS;
int HUM_IN_DAY_HIGHS;
int HUM_IN_DAY_HIGH_TIMES;
int HUM_IN_DAY_LOWS;
int HUM_IN_DAY_LOW_TIMES;
int HUM_IN_HOUR;
int HUM_IN_MONTH_HIGHS;
int HUM_IN_MONTH_LOWS;
int HUM_IN_YEAR_HIGHS;
int HUM_IN_YEAR_LOWS;
int HUM_OUT_DAY_HIGHS;
int HUM_OUT_DAY_HIGH_TIMES;
int HUM_OUT_DAY_LOWS;
int HUM_OUT_DAY_LOW_TIMES;
int HUM_OUT_HOUR;
int HUM_OUT_MONTH_HIGHS;
int HUM_OUT_MONTH_LOWS;
int HUM_OUT_YEAR_HIGHS;
int HUM_OUT_YEAR_LOWS;
int LEAF_DAY_HIGHS;
int LEAF_DAY_HIGH_TIMES;
int LEAF_DAY_LOWS;
int LEAF_DAY_LOW_TIMES;
int LEAF_HOUR;
int LEAF_MONTH_HIGHS;
int LEAF_MONTH_LOWS;
int LEAF_YEAR_HIGHS;
int LEAF_YEAR_LOWS;
int RAIN_15_MIN;
int RAIN_DAY_TOTAL;
int RAIN_HOUR;
int RAIN_MONTH_TOTAL;
int RAIN_RATE_1_MIN;
int RAIN_RATE_DAY_HIGHS;
int RAIN_RATE_DAY_HIGH_TIMES;
int RAIN_RATE_HOUR;
int RAIN_RATE_MONTH_HIGHS;
int RAIN_RATE_YEAR_HIGHS;
int RAIN_STORM;
int RAIN_STORM_END;
int RAIN_STORM_START;
int RAIN_YEAR_TOTAL;
int RX_PERCENTAGE;
int SAVE_DAY;
int SAVE_HOUR;
int SAVE_MIN;
int SAVE_MONTH;
int SAVE_YEAR;
int SAVE_YEAR_COMP;
int SOIL_DAY_HIGHS;
int SOIL_DAY_HIGH_TIMES;
int SOIL_DAY_LOWS;
int SOIL_DAY_LOW_TIMES;
int SOIL_HOUR;
int SOIL_MONTH_HIGHS;
int SOIL_MONTH_LOWS;
int SOIL_YEAR_HIGHS;
int SOIL_YEAR_HIGHS_COMP;
int SOIL_YEAR_LOWS;
int SOLAR_DAY_HIGHS;
int SOLAR_DAY_HIGH_TIMES;
int SOLAR_HOUR_AVG;
int SOLAR_MONTH_HIGHS;
int SOLAR_YEAR_HIGHS;
int TEMP_IN_DAY_HIGHS;
int TEMP_IN_DAY_HIGH_TIMES;
int TEMP_IN_DAY_LOWS;
int TEMP_IN_DAY_LOW_TIMES;
int TEMP_IN_HOUR;
int TEMP_IN_MONTH_HIGHS;
int TEMP_IN_MONTH_LOWS;
int TEMP_IN_YEAR_HIGHS;
int TEMP_IN_YEAR_LOWS;
int TEMP_OUT_DAY_HIGHS;
int TEMP_OUT_DAY_HIGH_TIMES;
int TEMP_OUT_DAY_LOWS;
int TEMP_OUT_DAY_LOW_TIMES;
int TEMP_OUT_HOUR;
int TEMP_OUT_MONTH_HIGHS;
int TEMP_OUT_MONTH_LOWS;
int TEMP_OUT_YEAR_HIGHS;
int TEMP_OUT_YEAR_LOWS;
int THSW_DAY_HIGHS;
int THSW_DAY_HIGH_TIMES;
int THSW_HOUR;
int THSW_MONTH_HIGHS;
int THSW_YEAR_HIGHS;
int UV_DAY_HIGHS;
int UV_DAY_HIGH_TIMES;
int UV_HOUR_AVG;
int UV_MEDS_DAY;
int UV_MEDS_HOUR;
int UV_MONTH_HIGHS;
int UV_YEAR_HIGHS;
int WIND_DIR_DAY;
int WIND_DIR_DAY_BINS;
int WIND_DIR_HOUR;
int WIND_DIR_MONTH;
int WIND_DIR_MONTH_BINS;
int WIND_SPEED_10_MIN_AVG;
int WIND_SPEED_DAY_HIGHS;
int WIND_SPEED_DAY_HIGH_DIR;
int WIND_SPEED_DAY_HIGH_TIMES;
int WIND_SPEED_HOUR_AVG;
int WIND_SPEED_HOUR_HIGHS;
int WIND_SPEED_MONTH_HIGHS;
int WIND_SPEED_MONTH_HIGH_DIR;
int WIND_SPEED_YEAR_HIGHS;
int WIND_SPEED_YEAR_HIGH_DIR;


#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __MAIN_H__ */
