/**********************************************************************************/
/*  vproweather       								  */
char		VersionM[] = "1.0.9";
static char	VerDate[] = "05-SEP-2013";
/*	A program for acquiring data from a Davis Vantage Pro Weather Station.	  */
/*										  */
/* (c)2004 Joe Jaworski    email: jj@joejaworski.com				  */
/* (c)2005-2013 Curt Blank email: curt@curtronics.com				  */
/* VPROWEATHER is covered under the GNU general license. See the attached	  */
/* LICENSE file for details.							  */
/*										  */
/* This software is provided as-is, without any expressed or implied		  */
/* warranties, including, without limitation, the implied warranties of		  */
/* merchantibility and fitness for any particular purpose.			  */
/*										  */
/* main.c - main entry and intitialization					  */
/*										  */
/*	v0.1 - initial revision							  */
/*	v0.2 - fixed baud rate setting problem, removed baud rate cmd line	  */
/*         parm because I don't have thhe time right now to track it down.	  */
/*	v0.3 - Added forecast rule strings					  */
/*	v0.4 - Fixed problem with string lookup					  */
/*	v0.5 - Fixed forecast strings. Fix baro trend problem.			  */
/*	v0.6 - Fix baro trend problem (again). Tweak parm checks.		  */
/*	v0.6-1 cjb - fixed _HUM_LO_BY_DAYS copying data for			  */
/*		     HUM_OUT_DAY_LOW_TIMES instead of HUM_OUT_DAY_LOW		  */
/*	v0.6-2 cjb - added and used bMtime in TimeConvert			  */
/*	v0.6-3 cjb - Don't bother using NEXT_10MIN_PTR or NEXT_15MIN_PTR	  */
/*		     they never get updated per Davis due to EEPROM update	  */
/*		     contraints						   	  */
/*	v0.6-4 cjb - print _BARO_CURR a.k.a rtBaroCurr out to 3 decimal places    */
/*		   - do the same for all _BARO_ attributes			  */
/*	v0.6-5 cjb - changed strftime, got rid of the commas for one thing	  */
/*	v0.6-6 cjb - added PrintSolarSet to output Solar graph data		  */
/*		   - do not divide Solar data by 10 (.i.e. not / 10.0)	   	  */
/*		   - added PrintETByteSet & PrintETSet to output ET graph data    */
/*		   - also output THSW data					  */
/*		   - added --dump-loop option					  */
/*     v0.6-7 cjb  - fix _WIND_DIR_ROSE (compass rose version) when testing for   */
/*                   >= 347 && < 12						  */
/*     v0.6-8 cjb  - convert hld.wTHSWHiDay using (signed short)                  */
/*                 - change THSW availibity check to hld.wTHSWHiDay == 65535      */
/*     v0.6-9 cjb  - modified graphic data sets-memory pointer offsets due        */
/*                   change in locations of the graph data in Vantage Pro2        */
/*     v0.7-0 cjb  - added -G --dump-graph packet option                          */
/*     v0.7-1 cjb  - we seemed to always have ignored WIND_SPEED_HOUR_HIGHS       */
/*                   (from the day of Joe) so let's add it now, no reason not to  */
/*     v0.7-2 cjb  - VP2 graph data bug fix                                       */
/*     v0.7-3 cjb  - Vantage Pro2 reports some null values as 32768 so zero them  */
/*     v0.7-4 cjb  - added WIND_HI_DIR_BY_DAYS                                    */
/*                 - added WIND_HI_DIR_BY_MONTHS                                  */
/*                 - added WIND_HI_DIR_BY_YEARS                                   */
/*                 - added UV_MEDS_BY_HOURS                                       */
/*                 - added UV_MEDS_BY_DAYS                                        */
/*     v0.7-5 cjb  - looks like VP2's offset RAIN_RATE_1_MIN by 32767, gonna      */
/*                   have to keep an eye on this                                  */
/*     v0.8.0 cjb  - add access to MySQL db                                       */
/*                 - now locks the serial port                                    */
/*     v0.8-14 cjb - change SubStrPos from int to pointer                         */
/*     v0.8-17 cjb - don't set lckCNT = -1 when clearing stale serial port lock   */
/*     v0.8-18 cjb - serial port configuration tweaks                             */
/*     v0.8-19 cjb - send a \n instead of a \r to wake up the console             */
/*                 - wait for \r wakeup                                           */
/*                 - wait up to 1.2 seconds between each attempt to wake up the   */
/*                   console per Davis doc                                        */
/*     v0.8-20 cjb - added errno for read problems and WakeUp write problems      */
/*     v0.8-22 cjb - added -w to select the number of wake up attempts to do      */
/*     v0.8-23 cjb - added -a when waking up the Vantage Pro it is suppose to     */
/*                   acknowledge with 0x0a0d but a lot of times it acknowledges   */
/*                   with 0x0a0a the -a options will accept the 0x0a0a as a       */
/*                   valid acknowledgment. This behavior was discovered on        */
/*                   Vantage Pro 2 but has been seen on the original Vantage Pro  */
/*                   also.                                                        */
/*     v0.8-24-39 cjb - lots of testing to try to get it working on a slug but    */
/*                      but can't seem to get it to wake up the console           */
/*     v0.8-40 cjb - correct memset sizeof passed pointer                         */
/*     v0.8-41 cjb - fix longopts not working and seg fault when compiled for     */
/*                   MySQL when using -fpack-struct but bogus data returned when  */
/*                   not compiled with -fpack-struct. No idea when this started   */
/*                   but can no longer compile with -fpack-struct which is        */
/*                   actually good from a performance standpoint.                 */
/*     v0.8-42 cjb - pack-struct problems appear to be solved                     */
/*                 - added -E option in SQL compile to prevent DB update          */ 
/*     v0.8-43 cjb - math format screw up in new HeatIndex calculation            */
/*     v0.8-44 cjb - dump the loop or graph raw data to the file if requested     */
/*                   before outputting said the data                              */
/*     v0.9.0  cjb - add option to get DMP & DMPAFT                               */
/*     v0.9.1  cjb - added dmpdConvertBE2LE routine for Little Endian             */
/*     v0.9.2  cjb - DMP & DMPAFT archive data was clobbering graph data when     */
/*                   run together, goes way back to buffer usage in v0.6 and      */
/*                   before, split into separate buffers                          */
/*                 - also, when real time data and archive data were requested    */
/*                   together archive data was not displayed, has been corrected  */
/*                   and is due to timing issue with getting LOOP packet          */
/*                 - report empty DMP & DMPAFT archive data records if so but     */
/*                   this may be bogus - based on user report                     */
/*     v0.9.3  cjb - correct date calculation used when sending the date for the  */
/*                   DMPAFT command, Davis had it wrong in the manual. !Suprise.  */
/*		   - entering 00000000-0000 for the DMPAFT date & time will       */
/*		     return all the data starting with the page that contains     */
/*		     the oldest record                                            */
/*		   - for some reason DMPAFT always starts with the record after   */
/*                   the record you ask for. It thinks that is the record you     */
/*                   asked for. Go figure. Maybe it's supposed to do that.        */
/*     v0.9.4  cjb - if a Little Endian machine convert DMPAFT pageCnt            */
/*     v0.9.5  cjb - for some odd reason do *not* Little Endian pageCnt bad idea  */
/*                   documentation says it's a WORD but it behaves more like a    */
/*                   BYTE                                                         */
/*                 - loop on some reads for up to 0.1 seconds, 3 seconds when     */
/*                   waiting for archive page count to give the console time to   */
/*                   respond this should improve relability and is necessary on   */
/*                   some units when archive data is requested along with other   */
/*                   data                                                         */
/*     v0.9.6  cjb - pass bRTDSuccess to OutputData                               */
/*     v0.9.8  cjb - add -W wait between character reads, mainly for serial over  */
/*                   Ethernet IP                                                  */
/*                 - add a 1 (one) second delay after writing before looking for  */
/*                   ACK in set time function                                     */
/*     v0.9.9  cjb - pass bGDSuccess to OutputData                                */
/*     v1.0.0  cjb - added getTZ function                                         */
/*                 - only grab the serial port if we really need it               */
/*                 - fixed ListOutputDataCols printing to stdout when it          */
/*                   shouldn't do that                                            */
/*                 - add -R option to print out vproweather's version             */
/*                 - put a 100uS wait between sending the command and reading the */
/*                   first char when getting the data structures. Thinking of     */
/*                   doing this for all write/reads to maybe improve reliability, */
/*                 - add -I option to try again to get -l, -g, -x, & -t data if   */
/*                   not successful on the first attempt for each option when the */
/*                   vPro is being obstainant                                     */
/*                 - no longer blow outta here just because we couldn't get       */
/*                   DavisTime (-t) but do report it as such                      */
/*                 - add -T option to get the time like -t but then check that it */
/*                   is equal to the system time and if not set it to the system  */
/*                   time                                                         */
/*     v1.0.1  cjb - had to add a sleep(1) when getting the model number now,     */
/*                   timing is becoming a PITA                                    */
/*                 - if the Lat or Lon is unknown and is needed try to get it     */
/*                   from the vPro                                                */
/*                 - change -L -dump-loop to -X -dump-loop because it dumps -x    */
/*                 - added -L --dump-hilo option                                  */
/*     v1.0.2  cjb - changed -E option from disable to enable                     */
/*                 - redo how MySQL options are picked up from ~/.my.cnf          */
/*                 - options -D, -S, -H, -P, -u, & -p override ~/.my.cnf          */
/*     v1.0.4  cjb - added -2 option for use with Vantage Pro 2 and removed       */
/*                   compile flag, now a runtime option                           */
/*     v1.0.5  cjb - no longer rely on MySQL for get_tty_password and             */
/*                   handle_options functions since some distros feel the need to */
/*                   strip it out thus not providing them                         */
/*                 - remove string length restrictions for -D, -H, -u, -p, & -S   */
/*                 - added CTRL-C trap to restore serial port setting and clean   */
/*                   up lock file                                                 */
/*                 - MySQL's ~/.my.cnf overrides options -D, -S, -H, -P, -u, & -p */
/*                 - change -F option to -q                                       */
/*                 - added new "-F, --format" option                              */
/*                 - removed MySQL password max length restriction when entering  */
/*                   password interactively                                       */
/*                 - -O option is not just for MySQL anymore but only comes into  */
/*                   play when either using MySQL or using the -F option          */
/*     v1.0.8  cjb - -o option has been changed to -n                             */
/*                 - -j option added to store data from multiple Vantage Pro      */
/*                   weather consoles in one database                             */
/*                 - -o & -i options added to export and import data from         */
/*                   multiple Vantage Pro weather consoles that may be at         */
/*                   different geographical locations into one database           */
/*     v1.0.9  cjb - times must be in 24 hour format to properly store the data   */
/*                   in the DB so need to force -M, --military-time just in case  */
/*                   it wasn't used when using -E, --enable-mysql                 */
/*										  */
/**********************************************************************************/
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <fcntl.h>
#include <termios.h>
#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include <unistd.h>
#include <math.h>
#include <string.h>
#include <getopt.h>
#include <time.h>
#include <errno.h>
//#include <error.h>
#include <signal.h>
#include "include/main.h"
#include "include/names.h"
#include "include/dhandler.h"
#ifndef noMySQL
#include <my_global.h>
#include <my_sys.h>
#include <mysql.h>
#include <my_getopt.h>
#include "include/mySQLhandler.h"
#endif
#ifdef __debug__
#include "include/debug.h"
#endif

long PID;

char RunTime[18] = " ";
char EndTime[18] = " ";

static char szCommandBuffer[12];
static char szResponseBuffer[12];
static char szLoopBuffer[101];
static char szHLBuffer[440];
static char szArcDataBuffer[269];
static char szSerDataBuffer[4200];			/* serial read/write buffer */
static int iLatitude = -1000.0;
static int iLongitude = -1000.0;

char rtDateTimeCT[20];
char hlDateTimeCT[20];
char grDateTimeCT[20];
time_t ttrt = 0;
time_t tthl = 0;
time_t ttgr = 0;

char* sFormat = NULL;

BOOL bVerbose = FALSE;				/* Verbose Mode */
BOOL bVpro2 = FALSE;
BOOL bGetRTD = FALSE;				/* Get Real Time Data */
BOOL bGetHLD = FALSE;				/* Get High Low Data */
BOOL bGetGD = FALSE;				/* Get graph data */
BOOL bMtime = FALSE;				/* use Military time */
BOOL bMetric = FALSE;
BOOL bTipMod = FALSE;
BOOL bGetDMP = FALSE;
BOOL bSTDOUT = FALSE;				/* Output to STDOUT flag */
BOOL bRTDSuccess = FALSE;
BOOL bHLDSuccess = FALSE;
BOOL bGDSuccess = FALSE;
BOOL bDMPSuccess = FALSE;
BOOL bCalcSun = FALSE;
BOOL bSolarNoonSec = FALSE;
BOOL bZcommas = FALSE;
BOOL bSwapEndian = FALSE;
double fLongitude = -1000.0;
double fLatitude = -1000.0;
#ifdef LATITUDE
    double latitude = LATITUDE;
#else
    double latitude = -1000.0;
#endif
#ifdef LONGITUDE
    double longitude = LONGITUDE;
#else
    double longitude = -1000.0;
#endif
char SunRise[26];
char SunSet[26];
char SolarNoon[11];
time_t ttRun = 0;
int dstRun = -1;

#ifndef noMySQL
BOOL bEnableMYSQL = FALSE;
BOOL bUseSQL = FALSE;				/* TRUE if we got a DB name */
BOOL bVerboseSQL = FALSE;			/* Verbose SQL Mode */
BOOL bQuiet = FALSE;				/* Surpress all SQL informational messages */
BOOL bProfiling = FALSE;
BOOL bImport = FALSE;
BOOL bExport = FALSE;
BOOL bImportOR = FALSE;
BOOL bConsoleIDset = FALSE;

char *opt_host_name = NULL;	/* server host (default=localhost) */
char *opt_db_name = NULL;	/* database name (default=none) */
char *opt_user_name = NULL;	/* username (default=login name) */
char *opt_password = NULL;	/* password (default=none) */
char *opt_socket_name = NULL;	/* socket name (use built-in value) */
static char *opt_host_name_s = NULL;
static char *opt_db_name_s = NULL;
static char *opt_user_name_s = NULL;
static char *opt_password_s = NULL;
static char *opt_socket_name_s = NULL;
unsigned int opt_port_num = 0;	/* port number (use built-in value) */
unsigned int opt_flags = 0;	/* connection flags (none) */
static int SQLport = -1;
char *sProfileDataFN = NULL;
char *sImportDataFN = NULL;
char *sExportDataFN = NULL;
char *sExportBegDate = NULL;
char *sExportEndDate = NULL;
int ConsoleID = 0;
static int GetParmsSQL(int argc, char *argv[]);

MYSQL *conn;                     /* pointer to connection handler */
const char *client_groups[] = { "vproweather", NULL };
#endif

int ClrSerLock(long PID);
void controlC(int sig);
BOOL toSTDOUT(BOOL success, BOOL format);

/* local Data */
static char szttyDevice[256];			/* Serial device string */
static BOOL bBKLOn = FALSE;			/* Backlite ON */
static BOOL bBKLOff = FALSE;			/* Backlite OFF */
static BOOL bVer = FALSE;			/* Davis firmware version astring */
static BOOL bVPwVer = FALSE;			/* vProWeather version */
static BOOL bModel = FALSE;			/* model name/number */
static BOOL bGetTime = FALSE;			/* Get time flag */
static BOOL bSetTime = FALSE;			/* Set time flag */
static BOOL bCorrectTime = FALSE;		/* Get and then set bSetTime if time is off */
static BOOL bWUresponse = FALSE;		/* May get 0x0a instead of 0x0d on wakeup, accept that */
static BOOL bWUsuppress = FALSE;		/* May get 0x0a instead of 0x0d on wakeup, suppress message */
static BOOL bHelp = FALSE;
static BOOL bListCols = FALSE;
static BOOL bConnect = FALSE;
static BOOL bConnectEE = FALSE;
static BOOL bDidntGetAll = FALSE;
static BOOL bTryTwice = FALSE;

static unsigned char yDelay;			/* Read wait time */
static unsigned int yReadCharWait = 0;		/* Read per char wait */
static unsigned int yWattempts = 3;		/* Number of attempts to wake up the Vantage Pro */
static char sGetDMPAFT[14];
static char sloopDumpFN[256];			/* file name to dump LOOP packet */
static char shiloDumpFN[256];			/* file name to dump HiLo packet */
static char sgraphDumpFN[256];			/* file name to dump graph packet */
static char sloopUseDumpFN[256];
static char sDataFN[256];			/* write certain parameters to a formatted file */
static int yLockWait = 0;			/* Seconds to wait to lock serial port */
static int wDMPPages = -1;
static char devLCKfile[256] = ttyLCKloc;
static char devLCKfileNew[256];
static char sEnvList[1024];			/* List of parameters to set as ENV variables */

/* local functions */
static int GetParms(int argc, char *argv[]);
static int WakeUp(int nfd, char wch);
static int ReadNextChar(int fdser, char *pChar);
static int ReadNextCharTO(int nfd, char *pChar, int timeout);
static void Delay(int secs, long microsecs);
static int ReadToBuffer(int fdser, char *pszBuffer, int nBufSize);
static int LockSerPort();
static float getTZ(int DST);
static char* getEERD(int fdser, char eeBuffer[], int bufsize, int address, char count);
static void getLatLon(int fdser);

/* options descriptor */
static struct option longopts[] = {
    { "vpro2",              no_argument,	0,	'2' },
    { "get-dmpaft",         required_argument,	0,	'A' },
    { "get-dmp",            optional_argument,	0,	'a' },
    { "Z-commas",           no_argument,	0,	'c' },
    { "metric",             optional_argument,	0,	'C' },
    { "delay",              required_argument,	0,	'd' },
    { "swap-endian",        no_argument,	0,	'e' },
    { "format",             required_argument,	0,	'F' },
    { "bklite-off",         no_argument,	0,	'f' },
    { "dump-graph",         required_argument,	0,	'G' },
    { "get-graph",          no_argument,	0,	'g' },
    { "help",               no_argument,	0,	'h' },
    { "latitude",           required_argument,	0,	'K' },
    { "longitude",          required_argument,	0,	'k' },
    { "dump-hilo",          required_argument,	0,	'L' },
    { "get-hilo",           no_argument,	0,	'l' },
    { "mtime",              no_argument,	0,	'M' },
    { "model",              no_argument,	0,	'm' },
    { "calc-sun",           optional_argument,	0,	'N' },
    { "set-env",            required_argument,	0,	'B' },  /* reserved but not enabled yet */
    { "bklite-on",          no_argument,	0,	'n' },
    { "vprowver",           no_argument,	0,	'R' },
    { "version",            no_argument,	0,	'r' },
    { "set-time",           no_argument,	0,	's' },
    { "trytwice",           no_argument,	0,	'I' },
    { "get-time",           no_argument,	0,	't' },
    { "get-set-time",       no_argument,	0,	'T' },
    { "use-loop-dump",      required_argument,	0,	'U' },
    { "verbose",            no_argument,	0,	'v' },
    { "read-wait",          required_argument,	0,	'W' },
    { "wakeup-attempts",    required_argument,	0,	'w' },
    { "dump-loop",          required_argument,	0,	'X' },
    { "get-realtime",       no_argument,	0,	'x' },
    { "0x0a-NO",            no_argument,	0,	'Y' },
    { "0x0a-OK",            no_argument,	0,	'y' },
    { "to-file",            required_argument,	0,	'Z' },
    { "list-columns",       no_argument,	0,	'z' },
    { "to-stdout",          no_argument,        0,      'O' },
#ifndef noMySQL
    { "enable-mysql",       no_argument,        0,      'E' },
    { "SQL-profile",        required_argument,  0,      'q' },
    { "quiet",              no_argument,        0,      'Q' },
    { "verboseSQL",         no_argument,        0,      'V' },
    { "database",           required_argument,	0,	'D' },
    { "host",               required_argument,	0,	'H' },
    { "port",               required_argument,	0,	'P' },
    { "password",           optional_argument,	0,	'p' },
    { "socket",             required_argument,	0,	'S' },
    { "user",               required_argument,	0,	'u' },
    { "consoleID",          required_argument,  0,      'j' },
    { "import",             required_argument,  0,      'i' },
    { "export",             required_argument,  0,      'o' },
#endif
    { 0,                    0,			0,	0 }
};


/*--------------------------------------------------------------------------
    toSTDOUT
    determine whether or not to output to stdout
----------------------------------------------------------------------------*/
BOOL toSTDOUT(BOOL success, BOOL format)
{
    if (sFormat == NULL) {
#ifndef noMySQL
        if (! bEnableMYSQL) {
#endif
            if (success) return(TRUE);
#ifndef noMySQL
        } else {
            if (success && bSTDOUT) return(TRUE);
        }
#endif
    } else
        if (success && (bSTDOUT || format)) return(TRUE);
    return(FALSE);
}


int fdser = -1;
struct termios oldtio;

/*--------------------------------------------------------------------------
    main
----------------------------------------------------------------------------*/
int main(int argc, char *argv[])
{
    struct termios newtio;
    int nCnt = 0;
    int RTDcnt, HLcnt, GDcnt, DMPcnt;
    int RC;
    struct tm stm;
    time_t tt;
    WORD pageCnt = 512;
    int i, seq, page, firstRec = 0, yDMPRety = 0;
    int dmpDay, dmpMonth, dmpYear, dmpHour, dmpMinute, dmpRec;
    WORD dmpDS, dmpTS;
    char ch;
    char *pos, *end;
    int yearRun = 0;
    int monthRun = 0;
    int dayRun = 0;
    int hourRun = 0;
    int minuteRun = 0;
    int secondRun = 0;
    int retries = 0;
    BOOL gotTime = FALSE;
#ifndef noMySQL
    static char *opt_password_t = NULL;
#endif

    PID = getpid();

    tt = ttRun = time(NULL);
    stm = *(localtime(&tt));
    dstRun = stm.tm_isdst;
    strftime(RunTime,18,"%G%m%d-%H:%M:%S ",&stm);
    yearRun = stm.tm_year + 1900;
    monthRun = stm.tm_mon + 1;
    dayRun = stm.tm_mday;
    hourRun = stm.tm_hour;
    minuteRun = stm.tm_min;
    secondRun = stm.tm_sec;

    /* Get command line parms */
    if (!GetParms(argc, argv) || bHelp) {
        fprintf(stderr, "\n\tThe Vantage Pro Swiss Army Knife\n\tFor Vantage Pro/Pro2");
#ifndef noMySQL
        fprintf(stderr, " with MySQL support");
#endif
        fprintf(stderr, "\n\n");
        fprintf(stderr, "Usage: vproweather [Options] Device\n");
        fprintf(stderr, "\nOptions:\n");
        fprintf(stderr, " -2, --vpro2                Using Vantage Pro 2 Weather Console\n");
        fprintf(stderr, " -x, --get-realtime         Get real time weather data\n");
        fprintf(stderr, " -l, --get-highlow          Get Highs/Lows data\n");
        fprintf(stderr, " -g, --get-graph            Get graph data\n");
        fprintf(stderr, " -A, --get-dmpaft=date      Get archive data after date yyyymmdd-hhmm (24 hour time format). An\n");
        fprintf(stderr, "                              Asterik (*) will indicate the requested record in the Sequence on\n");
        fprintf(stderr, "                              the first page. Entering 00000000-0000 will start with the oldest\n");
        fprintf(stderr, "                              data available.\n");
        fprintf(stderr, " -a, --get-dmp[=num]        Get \"num\" pages of archive data (Can be used with -A to limit pages.\n");
        fprintf(stderr, "                              \"num\" is optional, will get all pages if not specified.)\n");
        fprintf(stderr, " -t, --get-time             Get weather station time\n");
        fprintf(stderr, " -s, --set-time             Set weather station time to system time\n");
        fprintf(stderr, " -T, --get-set-time         Same as -t, then also set the time if it is incorrect\n");
        fprintf(stderr, " -n, --bklite-on            Turn backlite on\n");
        fprintf(stderr, " -f, --bklite-off           Turn backlite off\n");
        fprintf(stderr, " -r, --version              Query for Davis firmware version string\n");
        fprintf(stderr, " -R, --vprowver             show vProWeather version and exit\n");
        fprintf(stderr, " -X, --dump-loop=file       Dump the Real Time raw data packet to a file (Does not imply -x,\n");
        fprintf(stderr, "                              -x must be used to get the data in order to dump it.)\n");
        fprintf(stderr, " -L, --dump-hilo=file       Dump the hilo packet to a file (Does not imply -l, -l must be used\n");
        fprintf(stderr, "                              to get the data in order to dump it.)\n");
        fprintf(stderr, " -G, --dump-graph=file      Dump the graph packet to a file (Does not imply -g, -g must be used\n");
        fprintf(stderr, "                              to get the data in order to dump it.)\n");
        fprintf(stderr, " -M, --military-time        Display time in 24 hour format\n");
        fprintf(stderr, " -N, --calc-sun[=showsec]   Calculate Sunrise, Sunset, & Solar Noon times instead of using\n");
        fprintf(stderr, "                              VP data. (This adds Solar Noon which the VP does not provide.)\n");
        fprintf(stderr, "                              If \"showsec\" is included \"%s\" will include the seconds\n", _SOLAR_NOON);
        fprintf(stderr, "                              otherwise not. See the man page for more info on this option.\n");
        fprintf(stderr, " -K, --latitude=num         Latitude to use when calculating Sunrise, Sunset, and Solar Noon.\n");
#ifdef LATITUDE
        fprintf(stderr, "                            (Decimal notation default set to %f)\n", LATITUDE);
#else
        fprintf(stderr, "                            (Decimal notation i.e. 45.266972 - default not set)\n");
#endif
        fprintf(stderr, " -k, --longitude=num        Longitude to use when calculating Sunrise, Sunset, and Solar Noon.\n");
#ifdef LONGITUDE
        fprintf(stderr, "                            (Decimal notation default set to %f)\n", LONGITUDE);
#else
        fprintf(stderr, "                            (Decimal notation i.e. -88.056889 - default not set)\n");
#endif
        fprintf(stderr, " -m, --model                Query for weather station model name\n");
        fprintf(stderr, " -I, --trytwice             try twice each for -l, -g, -x, & -t to get the data if the first\n");
        fprintf(stderr, "                            attempt for each fails in case the vPro is being obstainant\n");
        fprintf(stderr, " -d, --delay=num            Cmd Delay in 1/10ths seconds. Default is 10 (1 sec)\n");
        fprintf(stderr, " -W, --read-wait=num        wait up to \"num\" mS between character reads (250 to 5000 mS)\n");
        fprintf(stderr, " -w, --wakeup-attempts=num  Default 3 attempts, minimum 2. Each attempt after the first\n");
        fprintf(stderr, "                              will wait an additional 1/2 second for the Vantage Pro to\n");
        fprintf(stderr, "                              wakeup so the 10th attempt will wait 4.5 seconds\n");
        fprintf(stderr, " -h, --help                 This message and exit\n");
        fprintf(stderr, " -Z, --to-file=filename     Output the data in columnar output to \"filename\" tab \"^I\"\n");
        fprintf(stderr, "                              deliminated. Requires -g & -x options for full data set. Data is\n");
        fprintf(stderr, "                              valid if line ends with [OK]\n");
        fprintf(stderr, " -z, --list-columns         Output header list of column names output by the -Z option. Will\n");
        fprintf(stderr, "                              output to file specified in -Z option if so specified. Will not\n");
        fprintf(stderr, "                              print in file if the first line in the file already contains the\n");
        fprintf(stderr, "                              header list. Can be used without supplying the serial port device\n");
        fprintf(stderr, "                              to get the column names.\n");
        fprintf(stderr, " -c, --Zcommas              Use commas \",\" instead of tabs to deliminate output in -Z option\n");
        fprintf(stderr, " -F, --format=string        Format Real Time data using \"string\" as format specifier or get\n");
        fprintf(stderr, "                              format specifier from file named in \"string\". For a list of data\n");
        fprintf(stderr, "                              items enter \"LIST\" as the \"string\" value. Automagically invokes\n");
        fprintf(stderr, "                              -x to get the Real Time data. See man page for details.\n");
        fprintf(stderr, " -e, --swap-endian          Swap Endianness\n");
        fprintf(stderr, " -C, --metric[=tipmod]      Convert data to Metric values. Note: If the Metric Measurement\n");
        fprintf(stderr, "                              Adapter is installed on the Tipping Bucket in the ISS include the\n");
        fprintf(stderr, "                              optional \"tipmod\" quailifer with this option. If using the short\n");
        fprintf(stderr, "                              option use \"1\" for installed \"0\" for not installed. (\"1\" & \"0\"\n");
        fprintf(stderr, "                              can also be used with long option)\n");
        fprintf(stderr, " -v, --verbose              Verbose mode.\n");
// Future		fprintf(stderr, " -n, --set-env         Set weather params as ENV vars (comma separated list)\n");
#ifndef noMySQL
        fprintf(stderr, "\nMySQL Options:\n");
        fprintf(stderr, " -E, --enable-mysql         Enable using MySQL DB. If not enabled -O option print to STDOUT is\n");
        fprintf(stderr, "                              automatically enabled.\n");
        fprintf(stderr, " -O, --to-stdout            Also print to STDOUT otherwise the data only will go into the MySQL\n");
        fprintf(stderr, "                              DB when the -E option is used\n");
        fprintf(stderr, " -H, --host=name            Host name (overridden by ~/.my.cnf)\n");
        fprintf(stderr, " -D, --database=name        DB name (overridden by ~/.my.cnf)\n");
        fprintf(stderr, " -u, --user=name            User name (overridden by ~/.my.cnf)\n");
        fprintf(stderr, " -p, --password[=secret]    Password (overridden by ~/.my.cnf)\n");
        fprintf(stderr, " -S, --socket=name          Socket path (overridden by ~/.my.cnf)\n");
        fprintf(stderr, " -P, --port=#               Port number (overridden by  ~/.my.cnf)\n");
        fprintf(stderr, " -Q, --quiet                Suppress all informational messages\n");
        fprintf(stderr, " -V, --verboseSQL           MySQL Verbose mode\n");
        fprintf(stderr, " -q, --SQL-profile=filename Output MySQL Performance Profile to \"filename\"\n");
        fprintf(stderr, " -j, --consoleID=num        Vantage Pro DB Console ID 0-999 Default 0 for \"Main DB\" (For storing\n");
        fprintf(stderr, "                              data from multiple weather stations in one MySQL DB) Must be unique\n");
        fprintf(stderr, "                              for each Vantage Pro Console\n");
        fprintf(stderr, " -i, --import=filename      Import data from another vproweather mySQL DB contained in\n");
        fprintf(stderr, "                              exported \"filename\"\n");
        fprintf(stderr, " -o, --export=<constraints> Export MySQL DB data to \"filename\" where constraints are\n");
        fprintf(stderr, "                              \"filename[:startDate[:endDate]]\" Date format YYYYMMDD\n");
        fprintf(stderr, "                              \"startDate\" is inclusive \"endDate\" is exclusive \"-j\" Vantage\n");
        fprintf(stderr, "                              Pro DB Console ID must be specified for export\n");
#endif
        fprintf(stderr, "\n Device                     Serial Port Device - Required parameter for operations that need\n");
        fprintf(stderr, "                                                 to communicate with the Vantage Pro.\n\n");
        fprintf(stderr, "Examples:\n");
        fprintf(stderr, "  vproweather --get-realtime /dev/ttyS0 > rtwdata.txt\n");
        fprintf(stderr, "      Gets real time data set to file rtwdata.txt from serial device ttyS0\n");
        fprintf(stderr, "  vproweather --verbose --bklite-on /dev/ttyS0\n");
        fprintf(stderr, "      Turns the LCD backlite On, illuminating the display.\n");
        fprintf(stderr, "\nvproweather v%-6s %s   MySQL Database can now store data for multiple Vantage Pro",VersionM,VerDate);
        fprintf(stderr, "\n                       Weather Consoles. Added Metric conversion, extensions, funtionality");
        fprintf(stderr, "\n                       and modifications, now supports Vantage Pro2 (graph data) and");
        fprintf(stderr, "\n                       different Endian architectures at run time. - by Curt Blank");
        fprintf(stderr, "\n                       (c)2005-%s Curt Blank email: weather at curtronics dot com ",VerDate+7);
        fprintf(stderr, "\n                       See http://www.curtronics.com/Weather/ for Weather Web site example");
        fprintf(stderr, "\n                       and the latest version and info\n\n");
        if (bHelp)
            exit(0);
        else
            exit(2);
    }

    if (bVPwVer) {
        fprintf(stdout, "v%s %s\n",VersionM,VerDate);
        exit(0);
    }

    if(bVerbose) {
        fprintf(stderr, "\nRunTime %s v%-6s for Vantage Pro",RunTime,VersionM);
        if (bVpro2) fprintf(stderr, " 2");
        fprintf(stderr, "\n");
    }

    (void) signal(SIGINT, controlC);
        
#ifndef noMySQL
    if (bVerboseSQL) showSQLopts();
    if (bEnableMYSQL) {
        if(bVerboseSQL) fprintf(stderr, "\nMySQL init bEnableMYSQL: %i bUseSQL %i\nMY_INIT argc %i\n",bEnableMYSQL,bUseSQL,argc);
        for (i = 1; i < argc; i++) argv[i] = '\0';
        argc = 1;
        MY_INIT (argv[0]);
        if(bVerboseSQL) fprintf(stderr, "load_defaults using group \'%s\' argc %i",client_groups[0],argc);
        load_defaults ("my", client_groups, &argc, &argv);
        if(bVerboseSQL) {
            fprintf (stderr, " done arguments argc %i\n",argc);
            for (i = 0; i < argc; i++) {
                if ((pos = strstr(argv[i],"--password=")) == NULL)
                    fprintf (stderr, "\targ %d: %s\n", i, argv[i]);
                else {
                    if (strlen(argv[i]-11) > 0) {
                        opt_password_t = getMemPtr(strlen(argv[i]-11));
                        strcpy(opt_password_t, pos+11);
                    }
                    fprintf (stderr, "\targ %d: --password=%s\n", i, maskPassword(opt_password_t));
                    if (opt_password_t != NULL ) free(opt_password_t);
                }
            }
            fprintf(stderr, "GetParmsSQL argc %i",argc);
        }
        GetParmsSQL(argc, argv);
        if(bVerboseSQL) fprintf(stderr, " done argc %i\n",argc);
        if (opt_host_name_s != NULL) opt_host_name = opt_host_name_s;
        if (opt_db_name_s != NULL) opt_db_name = opt_db_name_s;
        if (opt_user_name_s != NULL) opt_user_name = opt_user_name_s;
        if (opt_password_s != NULL) opt_password = opt_password_s;
        if (opt_socket_name_s != NULL) opt_socket_name = opt_socket_name_s;
        if (SQLport >= 0) opt_port_num = (unsigned int)SQLport;
    } else
        if(bVerboseSQL) fprintf(stderr, "\nMySQL not inited bEnableMYSQL: %i bUseSQL %i\n",bEnableMYSQL,bUseSQL);
    if (bVerboseSQL) showSQLopts();
#endif

    /* do this now because we may have to connect to the vPro to get the Lat & Lon if we have the serial port */
    if (bCalcSun) {
        if (fLatitude != -1000.0) latitude = fLatitude;
        if (fLongitude != -1000.0) longitude = fLongitude;
        if (! bConnect && strlen(szttyDevice) > 0 && (latitude ==  -1000.0 || longitude ==  -1000.0)) bConnectEE = TRUE;
    }


    if (bConnect || bConnectEE) {
        LockSerPort();

        if(bVerbose) fprintf(stderr, "Opening Serial Port %s...", szttyDevice);
        errno = 0;
        fdser = open(szttyDevice, O_RDWR | O_NOCTTY | O_NDELAY);
        if (errno != 0) fprintf (stderr, "\n\nvproweather: Problem opening serial device, %s.\n",strerror (errno));
        if (fdser < 0) {
            perror("vproweather: Problem opening serial device, check device name.");
            ClrSerLock(PID);
            exit(2);
        }
         if(bVerbose) fprintf(stderr, " Opened (%i)\n",fdser);

        tcgetattr(fdser, &oldtio);	/* save previous port settings */

        bzero(&newtio, sizeof(newtio));
        newtio.c_cflag &= ~PARENB;		/* no parity */
        newtio.c_cflag &= ~CSTOPB;		/* one stop bit */
        newtio.c_cflag &= ~CSIZE;		/* character size mask */
        newtio.c_cflag &= ~HUPCL;		/* no hangup */
        newtio.c_cflag |= CS8 | CLOCAL | CREAD;	/* 8 bit - ignore modem control lines - enable receiver */

        newtio.c_iflag = IGNBRK | IGNPAR;	/* ignore BREAK condition on input & framing errors & parity errors */

        newtio.c_oflag = 0;			/* set serial device input mode (non-canonical, no echo,...) */
        newtio.c_oflag &= ~OPOST;		/* disable output processing */

        newtio.c_lflag = 0;

        newtio.c_cc[VTIME]    = yDelay;		/* timeout in 1/10 sec intervals */
        newtio.c_cc[VMIN]     = 0;		/* block until char or timeout */
        cfsetospeed (&newtio, B19200);
        cfsetispeed (&newtio, B19200);
    
        if(tcsetattr(fdser, TCSANOW, &newtio)) {
            perror("vproweather: Problem configuring serial device, check device name.");
            ClrSerLock(PID);
            exit(2);
        }
        
        if(tcflush(fdser, TCIOFLUSH)) {
            perror("vproweather: Problem flushing serial device, check device name.");
            ClrSerLock(PID);
            exit(2);
        }

    
        /* wake up the station */
        if(bVerbose)
        fprintf(stderr, "\nWaking up weather station...");
        if(!WakeUp(fdser,VP_LF) && !WakeUp(fdser,VP_CR)) {
        fprintf(stderr, "vproweather: Can't wake up weather station - no response.\n");
        ClrSerLock(PID);
        exit(2);
        }	
    }

    
    /* ---------------------------- */
    /* Process Command line actions */
    /* ---------------------------- */
    
    /* turn on backlite */
    if(bBKLOn) {
        if(bVerbose) fprintf(stderr, "Turning Backlite ON...\nFlushing all I/O ");
        tcflush(fdser, TCIOFLUSH);
        strcpy(szCommandBuffer, "LAMPS 1\n");       /* make Davis cmd string */
        if(bVerbose) fprintf(stderr, "Writing I/O ");
        RC = write(fdser, &szCommandBuffer, strlen(szCommandBuffer)); /* send it */
        if (RC <= 0) fprintf(stderr, "\n WARNING: -o Problem sending command\n");
        if(bVerbose) fprintf(stderr, "Draining I/O ");
        tcdrain(fdser);
        if(bVerbose) fprintf(stderr, "Reading I/O ");
        nCnt = ReadToBuffer(fdser, szResponseBuffer, sizeof(szResponseBuffer));
        if(bVerbose) fprintf(stderr, "Got %d characters\n", nCnt);
        while ((pos = strrchr(szResponseBuffer, VP_CR)) || (pos = strrchr(szResponseBuffer, VP_LF))) *(pos) = VP_SP;
        pos = szResponseBuffer;
        while (*(pos) == VP_SP) pos++;
        end = pos;
        while (*(end) != VP_SP) end++;
        *(end) = '\0';
        printf("%s\n", pos);
    }
    
    /* turn off backlite */
    if(bBKLOff) {
        if(bVerbose) fprintf(stderr, "Turning Backlite OFF...\nFlushing all I/O ");
        tcflush(fdser, TCIOFLUSH);
        strcpy(szCommandBuffer, "LAMPS 0\n");       /* make Davis cmd string */
        if(bVerbose) fprintf(stderr, "Writing I/O ");
        RC = write(fdser, &szCommandBuffer, strlen(szCommandBuffer)); /* send it */
        if (RC <= 0) fprintf(stderr, "\n WARNING: -f Problem sending command\n");
        if(bVerbose) fprintf(stderr, "Draining I/O ");
        tcdrain(fdser);
        if(bVerbose) fprintf(stderr, "Reading I/O ");
        nCnt = ReadToBuffer(fdser, szResponseBuffer, sizeof(szResponseBuffer));
        if(bVerbose) fprintf(stderr, "Got %d characters\n", nCnt);
        while ((pos = strrchr(szResponseBuffer, VP_CR)) || (pos = strrchr(szResponseBuffer, VP_LF))) *(pos) = VP_SP;
        pos = szResponseBuffer;
        while (*(pos) == VP_SP) pos++;
        end = pos;
        while (*(end) != VP_SP) end++;
        *(end) = '\0';
        printf("%s\n", pos);
    }
    
    /* Get firmware version strings */ 
    if(bVer) {
        if(bVerbose) fprintf(stderr, "Getting firmware string...\n Flushing all I/O ");
        tcflush(fdser, TCIOFLUSH);
        strcpy(szCommandBuffer, "VER\n");	   /* make Davis cmd string */
        if(bVerbose) fprintf(stderr, "Writing I/O ");
        RC = write(fdser, &szCommandBuffer, strlen(szCommandBuffer)); /* send it */
        if (RC <= 0) fprintf(stderr, "\n WARNING: -r Problem sending command\n");
        if(bVerbose) fprintf(stderr, "Draining I/O ");
        tcdrain(fdser);

        if(bVerbose) fprintf(stderr, "Reading I/O\n");
        nCnt = ReadToBuffer(fdser, szResponseBuffer, sizeof(szResponseBuffer));
        if(bVerbose)
            fprintf(stderr, "Got %d characters\n", nCnt);
        *(szResponseBuffer+nCnt-2) = '\0';
        printf("\nVersion Info: %s\n", szResponseBuffer);
    }

    /* Get model number */ 
    if(bModel) {
        if(bVerbose) fprintf(stderr, "Getting model number code byte...\nFlushing all I/O ");
        tcflush(fdser, TCIOFLUSH);
        strcpy(szCommandBuffer, "WRD\022\115\n");   /* make Davis cmd string */
        if(bVerbose) fprintf(stderr, "Writing I/O ");
        RC = write(fdser, &szCommandBuffer, strlen(szCommandBuffer)); /* send it */
        if (RC <= 0) fprintf(stderr, "\n WARNING: -m Problem sending command\n");
        if(bVerbose) fprintf(stderr, "Draining I/O ");
        tcdrain(fdser);

        if(bVerbose) fprintf(stderr, "Reading I/O\n");
        if(!ReadNextCharTO(fdser, &ch, 1000000)) {
            fprintf(stderr, "vproweather: No ACK response to model number query (0x%02x)).\n",ch);
            ClrSerLock(PID);
            exit(2);
        }	
        if(bVerbose) fprintf(stderr, "Got 0x%02x\n",ch);
	sleep(1);

        if(!ReadNextChar(fdser, &ch)) {
            fprintf(stderr, "vproweather: Model number byte not sent.\n");
            ClrSerLock(PID);
            exit(2);
        }
        if(bVerbose) fprintf(stderr, "Got 0x%02x\n",ch);

        printf("\nModel: Davis ");		
        switch((int)ch) {
            case 0: printf("Wizard III\n");					break;
            case 1: printf("Wizard II\n");					break;
            case 2: printf("Monitor\n");					break;
            case 3: printf("Perception\n");					break;
            case 4: printf("GroWeather\n");					break;
            case 5: printf("Energy Environmonitor\n");		break;
            case 6: printf("Health Environmonitor\n");		break;
            case 16: printf("Vantage Pro\n");				break;
            default: printf("Unknown model (%d)\n", (int)ch);	break;
        }
        if(ch != 16)
            printf("Warning: This program may not work correctly with your model.\n");			
    }
    if(bVer || bModel) printf("\n"); 
    

    /* Get weather station time */ 
    if(bGetTime || bCorrectTime) {
        if(bVerbose) fprintf(stderr, "Getting weather station time...\nFlushing all I/O ");
        gotTime = FALSE;
        if (bTryTwice)
            retries = 1;
        else
            retries = 0;
        while (retries-- >= 0  && ! gotTime) {
            gotTime = FALSE;
            tcflush(fdser, TCIOFLUSH);
            strcpy(szCommandBuffer, "GETTIME\n");       /* make Davis cmd string */
            RC = write(fdser, &szCommandBuffer, strlen(szCommandBuffer)); /* send it */
            if (RC <= 0 && retries < 0) fprintf(stderr, "\n -t WARNING: Problem sending command\n");
            if(bVerbose) fprintf(stderr, "Reading: ");
            nCnt = i = 0;
            while (nCnt == 0  && i < 1000) {
                if (i == 0) usleep(100);
                nCnt = ReadToBuffer(fdser, szResponseBuffer, sizeof(szResponseBuffer));
                if (nCnt == 0) {
                    usleep(100);
                    i++;
                }
            }
            if(bVerbose) {
                fprintf(stderr, "Waited %d uS Got %d characters...",i*100,nCnt);
                if(nCnt != 9)
                    fprintf(stderr, "Bad\n");
                else
                    fprintf(stderr, "Good\n");
            }
            if(nCnt != 9) {
                fprintf(stderr, "vproweather: GETTIME - Didn't get all data. Try changing delay parameter. (%i/9)", nCnt);
                ClrSerLock(PID);
                i = 0;
                while (i < nCnt) {
                    fprintf(stderr, " 0x%02x",szResponseBuffer[i]);
                    i++;
                }
                if (retries < 0) 
                    fprintf(stderr, "\n");
                else 
                    fprintf(stderr, " Retrying...\n");
            } else {
                if((nCnt = CheckCRC(8, szResponseBuffer+1))) {	/* check crc */
                    fprintf(stderr,"vproweather: GETTIME - CRC failure %d.", nCnt);
                    if (retries < 0)
                        fprintf(stderr, "\n");
                     else
                        fprintf(stderr, " Retrying...\n");
                } else {
                    gotTime = TRUE;
                    if (bVerbose) fprintf(stderr, "CRC verified good on TIME packet.\n");
                }
            }
            if (gotTime && bTryTwice && retries < 0) fprintf(stderr,"vproweather: GETTIME - Second time success!\n");
        }
        if (! gotTime) {
            if (bVerbose) fprintf(stderr, "Couldn't get DavisTime as requested.\n");
        } else {
            PrintTime(szResponseBuffer);
            if (bCorrectTime && (yearRun != ((int)szResponseBuffer[6]+1900) || monthRun != (int)szResponseBuffer[5] || dayRun != (int)szResponseBuffer[4] || hourRun != (int)szResponseBuffer[3] || minuteRun != (int)szResponseBuffer[2])) {
                if (bVerbose) fprintf(stderr, "DavisTime incorrect\nyearRun %i\tszResponseBuffer[6] %i\nmonthRun  %2i\tszResponseBuffer[5]   %2i\ndayRun    %2i\tszResponseBuffer[4]   %2i\nhourRun   %2i\tszResponseBuffer[3]   %2i\nminuteRun %2i\tszResponseBuffer[2]   %2i\n", yearRun, (int)szResponseBuffer[6]+1900, monthRun, (int)szResponseBuffer[5], dayRun, (int)szResponseBuffer[4], hourRun, (int)szResponseBuffer[3], minuteRun, (int)szResponseBuffer[2]);
                fprintf(stderr, "DavisTime incorrect, flagging to correct as requested after data is gathered.\n");
                bSetTime = TRUE;
            }
        }
    }

    DMPcnt = 0;
    bDMPSuccess = FALSE;
    if(bGetDMP) {
        if(bVerbose) fprintf(stderr, "Getting archive data set...\nFlushing all I/O ");
        tcflush(fdser, TCIOFLUSH);
        strcpy(szCommandBuffer, "DMP\n");
        if(strlen(sGetDMPAFT) > 0) strcpy(szCommandBuffer, "DMPAFT\n");
        if(bVerbose) fprintf(stderr, "Sending command: %s",szCommandBuffer);
        RC = write(fdser, &szCommandBuffer, strlen(szCommandBuffer)); /* send it */
        if (RC <= 0) fprintf(stderr, "\n WARNING: -A Problem sending command\n");
        tcdrain(fdser);

        if (strlen(sGetDMPAFT) > 0) {
            sscanf(sGetDMPAFT, "%4d%2d%2d-%2d%2d", &dmpYear,&dmpMonth,&dmpDay,&dmpHour,&dmpMinute);	/* yyyymmdd-hhmm */
            if(bVerbose) fprintf(stderr, "Using %04d%02d%02d-%02d%02d\nReading: ",dmpYear,dmpMonth,dmpDay,dmpHour,dmpMinute);
            nCnt = DMPcnt = ReadToBuffer(fdser, szArcDataBuffer, sizeof(szArcDataBuffer));
            if (szArcDataBuffer[0] == VP_ACK) bDMPSuccess = TRUE;
            if(bVerbose) {
                fprintf(stderr, "Got %d characters", nCnt);
                if (nCnt >= 1) fprintf(stderr, " 0x%02x",szArcDataBuffer[0]);
                if (szArcDataBuffer[0] != VP_ACK) fprintf(stderr, " not");
                fprintf(stderr, " ACK\n");
            }
    /* The Vantage Pro and Vantage Pro2 Serial Support 2.2 - 01-25-2005 Document says that for this */
    /* calculation the year is offset by 1900,  but that is wrong, it is offset by 2000. Using 1900 */
    /* the console will not return data for the request  date on the first page like it is supposed */
    /* to. Using 2000 as the offset it works  correctly. I should know better the to believe Davis. */
            if (strcmp(sGetDMPAFT,"00000000-0000") == 0) {
                dmpDS = 0;
                dmpTS = 0;
            } else {
                dmpDS = (((dmpYear - 2000) << 9) & 0xFE00) + ((dmpMonth << 5) & 0x01E0) + dmpDay;
                dmpTS = (dmpHour*100) + dmpMinute;
            }
            szCommandBuffer[0] = LOBYTE(dmpDS);
            szCommandBuffer[1] = HIBYTE(dmpDS);
            szCommandBuffer[2] = LOBYTE(dmpTS);
            szCommandBuffer[3] = HIBYTE(dmpTS);
            i = CheckCRC(4, szCommandBuffer);                   /* generate CRC */
            szCommandBuffer[4] = HIBYTE(i);                     /* send MSB first */
            szCommandBuffer[5] = LOBYTE(i);

            if(bVerbose) {
                fprintf(stderr, "After %s %s 0x%04x 0x%04x\n",PrintDateDMP(dmpDS),TimeConvert(dmpTS),dmpDS,dmpTS);
                fprintf(stderr, "Sending date: 0x%02x 0x%02x 0x%02x 0x%02x CRC 0x%02x 0x%02x\n",(BYTE)szCommandBuffer[0],(BYTE)szCommandBuffer[1],(BYTE)szCommandBuffer[2],(BYTE)szCommandBuffer[3],(BYTE)szCommandBuffer[4],(BYTE)szCommandBuffer[5]);
            }
            tcflush(fdser, TCIOFLUSH);
            RC = write(fdser, &szCommandBuffer, 6); /* send it */
            if (RC <= 0) fprintf(stderr, "\n WARNING: -A(2) Problem sending command\n");
            tcdrain(fdser);
            if(bVerbose) fprintf(stderr, "Waiting up to 3 seconds Reading waiting ACK: ");
            nCnt = DMPcnt = i = 0;
            while (nCnt == 0  && i < 30000) {
                if (i == 0) usleep(100);
                nCnt = DMPcnt = ReadToBuffer(fdser, szArcDataBuffer, sizeof(szArcDataBuffer));
                if (nCnt == 0) {
                    usleep(100);
                    i++;
                }
            }
            if (szArcDataBuffer[0] == VP_ACK) bDMPSuccess = TRUE;
            if(bVerbose) {
                fprintf(stderr, "Waited %d uS Got %d characters",i*100,nCnt);
                if (nCnt > 0) fprintf(stderr, " 0x%02x",szArcDataBuffer[0]);
                if (szArcDataBuffer[0] != VP_ACK) fprintf(stderr, " not");
                fprintf(stderr, " ACK");
                if (szArcDataBuffer[0] == VP_CANCEL) fprintf(stderr, " bad CRC");
                if (! bDMPSuccess) fprintf(stderr, "\n");
            }
            pageCnt = -1;
            firstRec = -1;
            if(bDMPSuccess) {
                if(bVerbose) {
                    if(nCnt != 7)
                        fprintf(stderr, " Bad\n");
                     else
                        fprintf(stderr, " Good\n");
                }
                if(nCnt != 7) {
                    fprintf(stderr,"vproweather: DMP - Didn't get all data. Try changing delay parameter. (%i/7)\n", nCnt);
                    bDMPSuccess = FALSE;
                    bDidntGetAll = TRUE;
                }
                if((nCnt = CheckCRC(6, szArcDataBuffer+1))) {      /* check crc */
                    fprintf(stderr,"vproweather: CRC failure %d.\n", nCnt);
                    bDMPSuccess = FALSE;
                } else {
                /* for some odd reason do *not* Little Endian pageCnt - bad bad bad idea */
                    pageCnt = ((BYTE)szArcDataBuffer[2] << 8) + (BYTE)szArcDataBuffer[1];
                    firstRec = (BYTE)szArcDataBuffer[3];
                    if (bVerbose) fprintf(stderr, "CRC verified good on DMP page packet. Page count %d First Rec %d 0x%02x 0x%02x 0x%02x\n",pageCnt,firstRec,(BYTE)szArcDataBuffer[1],(BYTE)szArcDataBuffer[2],(BYTE)szArcDataBuffer[3]);
                    bDMPSuccess = TRUE;
                }
            }
        } else
            bDMPSuccess = TRUE;
        if(bDMPSuccess) {
            szCommandBuffer[0] = VP_ACK;
            seq = 0;
            page = 0;
            if (wDMPPages > 0 && wDMPPages < pageCnt) pageCnt = wDMPPages;
            while (bDMPSuccess && page < pageCnt) {
                if(bVerbose) {
                    fprintf(stderr, "Sending ");
                    switch (szCommandBuffer[0]) {
                        case VP_ACK: fprintf(stderr, "ACK "); break;
                        case VP_NAK: fprintf(stderr, "NAK "); break;
                        default: fprintf(stderr, "0x%02x ",szCommandBuffer[0]); break;
                    }
                }
                RC = write(fdser, &szCommandBuffer, 1); /* send it */
                if (RC <= 0) fprintf(stderr, "\n WARNING: -A(3) Problem sending command\n");
                tcdrain(fdser);
                if(bVerbose) fprintf(stderr, "Reading page %3d : ",page+1);
                nCnt = DMPcnt = ReadToBuffer(fdser, szArcDataBuffer, sizeof(szArcDataBuffer));
                if (nCnt == 268 && szArcDataBuffer[0] == VP_ACK) {
                    i = 0;
                    while (i < 267) {
                        szArcDataBuffer[i] = szArcDataBuffer[i+1];
                        i++;
                    }
                    nCnt = DMPcnt = 267;
                }
                if(bVerbose) {
                    fprintf(stderr, "Got %d characters...", nCnt);
                    if(DMPcnt != 267)
                        fprintf(stderr, "Bad\n");
                    else
                        fprintf(stderr, "Good\n");
                }
                if(DMPcnt != 267) {
                    fprintf(stderr,"vproweather: DMP Page - Didn't get all data. Try changing delay parameter. (%i/267)\n", nCnt);
                    bDMPSuccess = FALSE;
                    bDidntGetAll = TRUE;
                } else {
                    if((nCnt = CheckCRC(DMPcnt, szArcDataBuffer ))) {       /* check crc */
                        fprintf(stderr,"vproweather: CRC failure Got 0x%02x 0x%02x Calc 0x%02x 0x%02x.\n",szArcDataBuffer[DMPcnt-2],szArcDataBuffer[DMPcnt-1],HIBYTE(nCnt),LOBYTE(nCnt) );
                        szCommandBuffer[0] = VP_NAK;
                        yDMPRety++;
                        if (yDMPRety > 5) bDMPSuccess = FALSE;
                    } else {
                        yDMPRety = 0;
                        if (bVerbose) fprintf(stderr, "CRC verified good on DMP packet.\n");
                        dmpRec = 0;
                        while (dmpRec < 5) {
                            printf("\nPage %3d of %d Record %d\n",page+1,pageCnt,dmpRec+1);
                            GetDMPData(szArcDataBuffer+(dmpRec*52)+1);		 /* get data to struct */
                            PrintDMPData(seq,(BYTE)szArcDataBuffer[0],dmpRec+1,(dmpRec==firstRec)?" *":"");
                            if (dmpRec == firstRec) firstRec = -1;
                            dmpRec++;
                        }
                        if (page == 255 && dmpRec == 5) seq++;
                        if (bVerbose) fprintf(stderr, "\n");
                        szCommandBuffer[0] = VP_ACK;
                    }
                }
                page++;
            }
            if (!bVerbose) printf("\n");
            szCommandBuffer[0] = VP_ESC;	/* cancel */
            if(bVerbose) fprintf(stderr, "DMP/DMPAFT Sending cancel command: 0x%02x\n\n",szCommandBuffer[0]);
            RC = write(fdser, &szCommandBuffer, 1); /* send it */
            if (RC <= 0) fprintf(stderr, "\n WARNING: -A(4) Problem sending command\n");
            tcdrain(fdser);
        }
    }

    /* Get highs/lows data set */ 
    hlDateTimeCT[0] = '\0';
    HLcnt = 0;
    bHLDSuccess = FALSE;
    if(bGetHLD) {
        if(bVerbose) fprintf(stderr, "Getting highs/lows data set...\nFlushing all I/O ");
        gotTime = FALSE;
        if (bTryTwice)
            retries = 1;
        else
            retries = 0;
        while (retries-- >= 0  && ! bHLDSuccess) {
            tcflush(fdser, TCIOFLUSH);
            strcpy(szCommandBuffer, "HILOWS\n");			/* make Davis cmd string */
            RC = write(fdser, &szCommandBuffer, strlen(szCommandBuffer));	/* send it */
            if (RC <= 0 && retries < 0) fprintf(stderr, "\n WARNING: -l Problem sending command\n");
            tcdrain(fdser);
            tthl = DateTimeCT(hlDateTimeCT);
            hlDateTimeCT[strlen(hlDateTimeCT)] = '\0';
#ifndef noMySQL
            if(bVerbose || bVerboseSQL) {
                    fprintf(stderr, "\n");
#else
            if(bVerbose){
                fprintf(stderr, "\n");
#endif
                fprintf(stderr, "hlDateTimeCT = %s\ttthl = %llu\n", hlDateTimeCT,(unsigned long long)tthl);
            }
            /* we do this loop now to try to read for up to 0.1 second just in case we */
            /* did a DMP or DMPAFT because after doing either one of those the console */
            /* may not respond right away and this read would then fail                */
            nCnt = HLcnt = i = 0;
            while (nCnt == 0  && i < 1000) {
                if (i == 0) usleep(100);
                nCnt = HLcnt = ReadToBuffer(fdser, szHLBuffer, sizeof(szHLBuffer));
                if (nCnt == 0) {
                    usleep(100);
                    i++;
                }
            }
            if(bVerbose) {
                fprintf(stderr, "Waited %d uS Got %d characters...",i*100,nCnt);
                if(nCnt != 439)
                    fprintf(stderr, "Bad\n");
                else
                    fprintf(stderr, "Good\n");
            }
            if(nCnt != 439) {
                fprintf(stderr,"vproweather: HILOWS - Didn't get all data. Try changing delay parameter. (%i/439)", nCnt);
                if (retries < 0) {
                    bDidntGetAll = TRUE;
                    fprintf(stderr, "\n");
                } else
                    fprintf(stderr, " Retrying...\n");

            } else {
                if((nCnt = CheckCRC(438, szHLBuffer+1))) {	/* check crc */
                    fprintf(stderr,"vproweather: HILOWS - CRC failure %d.", nCnt);
                    if (retries < 0) {
                        bDidntGetAll = TRUE;
                        fprintf(stderr, "\n");
                     } else
                        fprintf(stderr, " Retrying...\n");
                } else {
                    if (bVerbose) fprintf(stderr, "CRC verified good on HILOWS packet. (%d)\n", HLcnt);
                    GetHLData(szHLBuffer);			/* get data to struct */
                    bHLDSuccess = TRUE;
                }
            }
            if (bHLDSuccess && bTryTwice && retries < 0) fprintf(stderr,"vproweather: HILOWS - Second time success!\n");
        }
    }	


    /* Get Graph data sets */
    grDateTimeCT[0] = '\0';
    GDcnt = 0;
    bGDSuccess = FALSE;
    if(bGetGD) {
        if(bVerbose) fprintf(stderr, "Getting graph data set...\nFlushing all I/O ");
        if (bTryTwice)
            retries = 1;
        else
            retries = 0;
        while (retries-- >= 0  && ! bGDSuccess) {
            tcflush(fdser, TCIOFLUSH);
            strcpy(szCommandBuffer, "GETEE\n");		 /* make Davis cmd string */
            RC = write(fdser, &szCommandBuffer, strlen(szCommandBuffer)); /* send it */
            if (RC <= 0 && retries < 0) fprintf(stderr, "\n WARNING: -g Problem sending command\n");
            tcdrain(fdser);
            ttgr = DateTimeCT(grDateTimeCT);
            grDateTimeCT[strlen(grDateTimeCT)] = '\0';
#ifndef noMySQL
            if(bVerbose || bVerboseSQL) {
                if (bVerbose)
                    fprintf(stderr, "\n");
#else
            if(bVerbose){
                fprintf(stderr, "\n");
#endif
                fprintf(stderr, "grDateTimeCT = %s\tttgr = %llu\n", grDateTimeCT,(unsigned long long)ttgr);
            }
            nCnt = GDcnt = i = 0;
            while (nCnt == 0  && i < 1000) {
                if (i == 0) usleep(100);
                nCnt = GDcnt = ReadToBuffer(fdser, szSerDataBuffer, sizeof(szSerDataBuffer));
                if (nCnt == 0) {
                    usleep(100);
                    i++;
                }
            }
            if(bVerbose) {
                fprintf(stderr, "Waited %d uS Got %d characters...",i*100,nCnt);
                if(nCnt != 4099)
                    fprintf(stderr, "Bad\n");
                else
                    fprintf(stderr, "Good\n");
            }
            if(nCnt != 4099) {
                fprintf(stderr, "vproweather: GETEE - Didn't get all data. Try changing delay parameter. (%i/4099)", nCnt);
                if (retries < 0) {
                    bDidntGetAll = TRUE;
                    fprintf(stderr, "\n");
                } else
                    fprintf(stderr, " Retrying...\n");
            } else {
                if((nCnt = CheckCRC(4098, szSerDataBuffer+1))) {	/* check crc */
                    fprintf(stderr, "vproweather: GETEE - CRC failure %d.", nCnt);
                    if (retries < 0) {
                        bDidntGetAll = TRUE;
                        fprintf(stderr, "\n");
                     } else
                        fprintf(stderr, " Retrying...\n");
                } else {
                    if (bVerbose) fprintf(stderr, "CRC verified good on full EEPROM packet. (%d)\n", GDcnt);
                    bGDSuccess = TRUE;
                }
            }
            if (bGDSuccess && bTryTwice && retries < 0) fprintf(stderr,"vproweather: GETEE - Second time success!\n");
        }
    }

    if (bCalcSun) {
        if(bVerbose) fprintf(stderr, "\nCalculating Sunrise, Sunset, & SolarNoon\n");
        if (latitude == -1000.0 || longitude == -1000.0) {
            if (bGDSuccess) {
                iLatitude = *( (WORD*)(szSerDataBuffer + EE_LATITUDE) );
                iLongitude = *( (WORD*)(szSerDataBuffer + EE_LONGITUDE) );
            } else
                getLatLon(fdser);
            if (iLatitude & 0x8000) {
                iLatitude =  iLatitude ^ 0xffff;
                iLatitude =  -iLatitude;
            }
            if (iLongitude & 0x8000) {
                iLongitude = iLongitude ^ 0xffff;
                iLongitude = -iLongitude;
            }
            latitude = iLatitude / 10.0;
            longitude = iLongitude / 10.0;
        }
        if (latitude != -1000.0 && longitude!= -1000.0) {
            calcSunRiseSetNoon(latitude, longitude, yearRun, monthRun, dayRun, hourRun, minuteRun, secondRun, dstRun, getTZ(dstRun));
            if (!bConnect)
                printf("\nSunrise %s Sunset %s SolarNoon %s\n\n", SunRise, SunSet, SolarNoon);
            else
                if(bVerbose) fprintf(stderr, "Sunrise %s Sunset %s SolarNoon %s\n\n", SunRise, SunSet, SolarNoon);
        } else {
        fprintf(stderr, "Can't determine Sunrise/Sunset");
        if (latitude == -1000.0) fprintf(stderr, " - Don't know Latitude");
        if (longitude == -1000.0) fprintf(stderr, "- Don't know Longitude");
        fprintf(stderr, "\n");
        }
        if (bConnectEE && ! bConnect) {
            /* restore previous port settings */
            if (tcsetattr(fdser, TCSANOW, &oldtio)) perror("vproweather: Problem resetting serial device configuration.");
            if(close(fdser)) perror("vproweather: Problem closing serial device, check device name.");
            ClrSerLock(PID);
        }
    }

    /* Get real time data set (Davis LOOP data) */ 
    rtDateTimeCT[0] = '\0';
    RTDcnt = 0;
    bRTDSuccess = FALSE;
    if(bGetRTD) {
        if(bVerbose) fprintf(stderr, "Getting real time data set...\nFlushing all I/O ");
        if (bTryTwice)
            retries = 1;
        else
            retries = 0;
        while (retries-- >= 0  && ! bRTDSuccess) {
            tcflush(fdser, TCIOFLUSH);
            strcpy(szCommandBuffer, "LOOP 1\n");
            if(bVerbose) fprintf(stderr, "Sending command: %s",szCommandBuffer);
            RC = write(fdser, &szCommandBuffer, strlen(szCommandBuffer)); /* send it */
            if (RC <= 0 && retries < 0) fprintf(stderr, "\n WARNING: -x Problem sending command\n");
            tcdrain(fdser);
            ttrt = DateTimeCT(rtDateTimeCT);
            rtDateTimeCT[strlen(rtDateTimeCT)] = '\0';
#ifndef noMySQL
            if(bVerbose || bVerboseSQL)
#else
            if(bVerbose)
#endif
                fprintf(stderr, "rtDateTimeCT = %s\tttrt = %llu\n", rtDateTimeCT,(unsigned long long)ttrt);
            if(bVerbose) fprintf(stderr, "Reading: ");
            nCnt = RTDcnt = i = 0;
            while (nCnt == 0  && i < 1000) {
                if (i == 0) usleep(100);
                nCnt = RTDcnt = ReadToBuffer(fdser, szLoopBuffer, sizeof(szLoopBuffer));
                if (nCnt == 0) {
                    usleep(100);
                    i++;
                }
            }
            if(bVerbose) {
                fprintf(stderr, "Waited %d uS Got %d characters...",i*100,nCnt);
                if(nCnt != 100)
                    fprintf(stderr, "Bad\n");
                else
                    fprintf(stderr, "Good\n");
            }
            if(nCnt != 100) {
                fprintf(stderr, "vproweather: LOOP - Didn't get all data. Try changing delay parameter. (%i/100)", nCnt);
                if (retries < 0) {
                    bDidntGetAll = TRUE;
                    fprintf(stderr, "\n");
                } else
                    fprintf(stderr, " Retrying...\n");
            } else {
                if((nCnt = CheckCRC(99, szLoopBuffer+1))) {	/* check crc */
                    fprintf(stderr, "vproweather: LOOP - CRC failure %d.", nCnt);
                    if (retries < 0) {
                        bDidntGetAll = TRUE;
                        fprintf(stderr, "\n");
                     } else
                        fprintf(stderr, " Retrying...\n");
                } else {
                    if (bVerbose) fprintf(stderr, "CRC verified good on LOOP packet. (%d)\n", RTDcnt);
                    GetRTData(szLoopBuffer);				/* get data to struct */
                    bRTDSuccess = TRUE;
                }
            }
            if (bRTDSuccess && bTryTwice && retries < 0) fprintf(stderr,"vproweather: LOOP - Second time success!\n");
        }
    }

    if (strlen(sloopUseDumpFN) > 0) {
        RTDcnt = GetData(szLoopBuffer, 100, sloopUseDumpFN);
        if (RTDcnt == 100) {
            GetRTData(szLoopBuffer);
            bRTDSuccess = TRUE;
        } else {
            bDidntGetAll = TRUE;
            fprintf(stderr, "\nvproweather: LOOP - Didn't get all data from %s. (%i/100)\n\n", sloopUseDumpFN, RTDcnt);
        }
    }

    /* Set weather station time */
    if(bSetTime) {
        if (bCorrectTime) sleep(1);
        if(bVerbose) fprintf(stderr, "Setting weather station time...\nFlushing all I/O ");
        tcflush(fdser, TCIOFLUSH);
        strcpy(szCommandBuffer, "SETTIME\n");   /* make Davis cmd string */
        RC = write(fdser, &szCommandBuffer, strlen(szCommandBuffer)); /* send it */
        if (RC <= 0) fprintf(stderr, "\n WARNING: -s Problem sending command\n");
        Delay(1, 0L);                           /* try to read too quick it will fail */
        ReadNextChar(fdser, &ch);               /* get ACK */
        if(ch != VP_ACK) {
            fprintf(stderr, "vproweather: Failed to get ACK.\n");
            ClrSerLock(PID);
            exit(2);
        }
        else if(bVerbose)
            fprintf(stderr, "Received ACK, sending system time...\n");

        time(&tt);
        stm = *localtime(&tt);                          /* get system time */
        stm.tm_mon  += 1;

        szCommandBuffer[0] = (char)stm.tm_sec;
        szCommandBuffer[1] = (char)stm.tm_min;
        szCommandBuffer[2] = (char)stm.tm_hour;
        szCommandBuffer[3] = (char)stm.tm_mday;
        szCommandBuffer[4] = (char)stm.tm_mon;
        szCommandBuffer[5] = (char)stm.tm_year;
        i = CheckCRC(6, szCommandBuffer);               /* generate CRC */
        szCommandBuffer[6] = HIBYTE(i);                 /* send MSB first */
        szCommandBuffer[7] = LOBYTE(i);

        if(bVerbose) fprintf(stderr, "Flushing all I/O ");
        tcflush(fdser, TCIOFLUSH);
        RC = write(fdser, &szCommandBuffer, strlen(szCommandBuffer)); /* send it */
        if (RC <= 0) fprintf(stderr, "\n WARNING: -s(2) Problem sending command\n");
        tcdrain(fdser);
        Delay(1, 0L);

        ReadNextChar(fdser, &ch);                       /* get ACK from console */
        if(ch != VP_ACK) {
            fprintf(stderr, "vproweather: CRC Failed, didn't get ACK (%d)\n", ch);
            ClrSerLock(PID);
            exit(2);
        }
        else if(bVerbose)
            fprintf(stderr, "Received ACK, CRC good, time is now set.\n");
    }

    if (bConnect) {
        /* restore previous port settings */
        if (tcsetattr(fdser, TCSANOW, &oldtio)) perror("vproweather: Problem resetting serial device configuration.");
        if(close(fdser)) perror("vproweather: Problem closing serial device, check device name.");
        ClrSerLock(PID);
    }

    if(bRTDSuccess && (bGetRTD ||strlen(sloopUseDumpFN) > 0)) {	/* If we successfully got the Real time data set */
        if (bVerbose || strlen(sloopUseDumpFN) > 0) {
            fprintf(stderr, "Processing Realtime data");
            if (strlen(sloopUseDumpFN) > 0) fprintf(stderr, " from file %s",sloopUseDumpFN);
            fprintf(stderr, "\n");
        }
        if (strlen(sloopDumpFN) > 0) DumpData(szLoopBuffer, RTDcnt, sloopDumpFN);
        if (sFormat != NULL)
            if (toSTDOUT(bRTDSuccess, TRUE)) PrintRTDataFmt(sFormat);
        if (toSTDOUT(bRTDSuccess, FALSE)) PrintRTData();			/* Real time data set to stdout */
    }

    if(bGetHLD && bHLDSuccess) {			/* If we successfully got the Highs/Lows data set */
        if (bVerbose) fprintf(stderr, "Processing High/Low data\n");
        if (strlen(shiloDumpFN) > 0) DumpData(szHLBuffer, HLcnt, shiloDumpFN);
        if (toSTDOUT(bHLDSuccess, FALSE)) PrintHLData();			/* Highs/Lows data set to stdout */
    }


    if(bGetGD && bGDSuccess) {				/* If we successfully got the Graph data sets */
        if (bVerbose) fprintf(stderr, "Processing Graph data\n");
        if (strlen(sgraphDumpFN) > 0) DumpData(szSerDataBuffer, GDcnt, sgraphDumpFN);
        if (toSTDOUT(bGDSuccess, FALSE)) PrintGDData(szSerDataBuffer);	/* Graph data sets to stdout */
    }

    if (bListCols) {
        if (bConnect && strlen(sDataFN) > 0)
            ListOutputDataCols(sDataFN);
        else {
            fprintf( stdout, "\n");
            ListOutputDataCols("");
            fprintf( stdout, "\n");
        }
    }

    if (bConnect && strlen(sDataFN) > 0) OutputData(szSerDataBuffer, sDataFN, bRTDSuccess, bGDSuccess);

#ifndef noMySQL
    if(bVerboseSQL) fprintf(stderr, "bEnableMYSQL = %s bUseSQL = %s ", bEnableMYSQL ? "TRUE" : "FALSE", bUseSQL ? "TRUE" : "FALSE");
    if (bUseSQL && bEnableMYSQL && opt_db_name != NULL && (bRTDSuccess || bHLDSuccess || bGDSuccess)) {
        if(bVerboseSQL) {
            fprintf(stderr, "Calling mySQLprocess...\n\t");
            fprintf(stderr, "%s  ", bRTDSuccess ? "bRTDSuccess = TRUE" : bGetRTD ? "bRTDSuccess = FALSE" : "bGetRTD = FALSE");
            fprintf(stderr, "%s  ", bHLDSuccess ? "bHLDSuccess = TRUE" : bGetHLD ? "bHLDSuccess = FALSE" : "bGetHLD = FALSE");
            fprintf(stderr, "%s\n\n", bGDSuccess ? "bGDSuccess = TRUE" : bGetGD ? "bGDSuccess = FALSE" : "bGetGD = FALSE");
            
        }
        mySQLprocess(szSerDataBuffer, ConsoleID);
    } else
        if(bVerboseSQL) {
            fprintf(stderr, "Not calling mySQLprocess...\n");
            if (bUseSQL && bEnableMYSQL) {
                fprintf(stderr, "\t%s  ", bRTDSuccess ? "bRTDSuccess = TRUE" : bGetRTD ? "bRTDSuccess = FALSE" : "bGetRTD = FALSE");
                fprintf(stderr, "%s  ", bHLDSuccess ? "bHLDSuccess = TRUE" : bGetHLD ? "bHLDSuccess = FALSE" : "bGetHLD = FALSE");
                fprintf(stderr, "%s\n", bGDSuccess ? "bGDSuccess = TRUE" : bGetGD ? "bGDSuccess = FALSE" : "bGetGD = FALSE");
            }
        }

    if (strlen(sEnvList) > 0) SetEnvVar(szSerDataBuffer, sEnvList);

    if (bImport) doImport();
    if (bExport) doExport(ConsoleID);
#endif

#ifdef __debug__
    if (bVerbose)
        PrintGraphOffsets();
#endif

    if (bVerbose) {
        tt = ttRun = time(NULL);
        stm = *(localtime(&tt));
        strftime(EndTime,18,"%G%m%d-%H:%M:%S ",&stm);
        fprintf(stderr, "\nAll done exiting %s...\n\n",EndTime);
    }

    free(sFormat);
#ifndef noMySQL
    free(opt_password_s);
#endif

    if (bDidntGetAll)
        exit(3);
    else
        exit(0);
}


/*--------------------------------------------------------------------------
    controlC
    CTRL-C trap
----------------------------------------------------------------------------*/
void controlC(int sig)
{
   fprintf(stderr, "\nCaught Ctrl-C\n");
   if(fdser >= 0) {
       if(bVerbose) fprintf(stderr, "Restoring serial device configuration\n");
       if (tcsetattr(fdser, TCSANOW, &oldtio)) perror("vproweather: Problem resetting serial device configuration.");
       if(bVerbose) fprintf(stderr, "Closing serial port\n");
       if (close(fdser)) perror("vproweather: Problem closing serial device.");
   }
   ClrSerLock(PID);
   fprintf(stderr, "\n");
   (void) signal(SIGINT, SIG_DFL);
   exit(2);
}

/*--------------------------------------------------------------------------
    getEERD
    Get EEPROM data
----------------------------------------------------------------------------*/
char* getEERD(int fdser, char eeBuffer[], int bufsize, int address, char count)
{
    int retries = -1;	/* placeholder */
    int i, nCnt, RC;

    eeBuffer[bufsize-1] = '\0';

    tcflush(fdser, TCIOFLUSH);
    sprintf(szCommandBuffer, "EERD %x %x\n", address, count);	/* make Davis cmd string */
    if(bVerbose) {
        fprintf(stderr, "Command: ");
        for (i = 0; i < strlen(szCommandBuffer); i++) {
            fprintf(stderr, "%02x ",(unsigned char)szCommandBuffer[i]);
        }
        fprintf(stderr, "\nWriting I/O ");
    }
    RC = write(fdser, &szCommandBuffer, strlen(szCommandBuffer)); /* send it */
    if (RC <= 0) fprintf(stderr, "\n WARNING: EERD Problem sending command\n");
    if(bVerbose) fprintf(stderr, "Draining I/O ");
    tcdrain(fdser);

    nCnt = i = 0;
    while (nCnt == 0  && i < 200) {
        if (i == 0) usleep(100);
        nCnt = ReadToBuffer(fdser, eeBuffer, bufsize);
        if (nCnt == 0) {
            usleep(100);
            i++;
        }
    }
    if(bVerbose) {
        fprintf(stderr, "\nResponse: ");
        for (i = 0; i < strlen(eeBuffer); i++) {
            fprintf(stderr, "%02x ",(unsigned char)eeBuffer[i]);
        }
        fprintf(stderr, "\nWaited %d uS Got %d characters...",i*100,nCnt);
        if(nCnt != bufsize-1)
            fprintf(stderr, "Bad\n");
        else
            fprintf(stderr, "Good\n");
    }
    if(nCnt != bufsize-1) {
        fprintf(stderr,"vproweather: LatLon - Didn't get all data. Try changing delay parameter. (%i/%i)", nCnt, bufsize-1);
        if (retries < 0) {
            bDidntGetAll = TRUE;
            fprintf(stderr, "\n");
        } else
            fprintf(stderr, " Retrying...\n");
    }
    return eeBuffer;
}

/*--------------------------------------------------------------------------
    getLatLon
    Get the Latitude & Longitude as integers
----------------------------------------------------------------------------*/
void getLatLon(int fdser)
{
    char LatLonBuffer[15];
    char cLat[5], cLon[5];

    if(bVerbose) fprintf(stderr, "Getting Latitude & Longitude from Weather Station...\n");

    strcpy(LatLonBuffer, getEERD(fdser, LatLonBuffer, sizeof(LatLonBuffer), VP_LAT, 2));
    cLat[0] = LatLonBuffer[10];
    cLat[1] = LatLonBuffer[11];
    cLat[2] = LatLonBuffer[6];
    cLat[3] = LatLonBuffer[7];
    cLat[4] = '\0';

    strcpy(LatLonBuffer, getEERD(fdser, LatLonBuffer, sizeof(LatLonBuffer), VP_LON, 2));
    cLon[0] = LatLonBuffer[10];
    cLon[1] = LatLonBuffer[11];
    cLon[2] = LatLonBuffer[6];
    cLon[3] = LatLonBuffer[7];
    cLon[4] = '\0';

    sscanf(cLat, "%X", &iLatitude);
    sscanf(cLon, "%X", &iLongitude);

    if(bVerbose) fprintf(stderr,"LatLonBuffer: %s 0x%04x %s 0x%04x\n", cLat, iLatitude, cLon, iLongitude);

    return;
}

/*--------------------------------------------------------------------------
    getTZ
    Get the TimeZone and return it formatted in decimal notation
----------------------------------------------------------------------------*/
float getTZ(int DST)
{
    time_t tt;
    struct tm stm;
    char cTZ[6];
    float TZ = 0.0;
    int iTZ = 0;
    int hours = 0;
    int minutes = 0;

    cTZ[0] = '\0';
    time(&tt);
    stm = *localtime(&tt);
    strftime(cTZ, sizeof(cTZ), "%z", &stm);
    iTZ = atoi(cTZ);
    hours = iTZ/100;
    minutes = iTZ - (hours*100);
    if (DST) hours--;
    TZ = (double)hours + ((double)minutes/60.0);
    return TZ;
}

/*--------------------------------------------------------------------------
    ClrSerLock
    Clear Serial Port lock.
----------------------------------------------------------------------------*/
int ClrSerLock(long PID) {
    FILE *fdserlck, *fdserlcknew;
    long rPID;
    int bWrite, bRead;

    if (strcmp(devLCKfile,ttyLCKloc) == 0) {
        if(bVerbose) fprintf(stderr, "devLCKfile <%s> was not initialized\n",devLCKfile);
        return(-1);
    }

    if (! bConnect) {
        if(bVerbose) fprintf(stderr, "ClrSerLock: connection not requested\n");
        return(-1);
    }

    if(bVerbose) fprintf(stderr, "\ndevLCKfile <%s>\ndevLCKfileNew <%s>\nClearing Serial Port Lock (%ld)...", devLCKfile, devLCKfileNew, PID);
    fdserlck = fopen(devLCKfile, "r");
    if (fdserlck == NULL) {
	if(bVerbose) fprintf(stderr, "\n");
	fprintf(stderr, "%s: %s: Problem opening serial device lock file to clear PID %ld: %s for read.\n\n",RunTime,ProgramName,PID,devLCKfile);
	return(0);
    }
    fdserlcknew = fopen(devLCKfileNew, "w");
    if (fdserlcknew == NULL) {
	if(bVerbose) fprintf(stderr, "\n");
	fprintf(stderr, "%s: %s: Problem opening new serial device lock file to clear PID %ld: %s for write.\n\n",RunTime,ProgramName,PID,devLCKfileNew);
	fclose(fdserlck);
	return(0);
    }
    bRead = fscanf(fdserlck, "%ld", &rPID);
    if (bRead <= 0 && bRead != EOF) fprintf(stderr, "\n WARNING: Problem writing lock file\n");
    while (bRead != EOF) {
	if (rPID != PID) {
            bWrite = fprintf(fdserlcknew, "%ld\n", rPID);
            if (bWrite <= 0) fprintf(stderr, "\n WARNING: Problem writing lock file\n");
        }
	bRead = fscanf(fdserlck, "%ld", &rPID);
        if (bRead <= 0 && bRead != EOF) fprintf(stderr, "\n WARNING: Problem writing lock file\n");
    }
    fclose(fdserlck);
    fclose(fdserlcknew);
    rename(devLCKfileNew,devLCKfile);
    if(bVerbose) fprintf(stderr, " done.\n\n");

    return(-1);
}


/*--------------------------------------------------------------------------
    LockSerPort
    Get a lock on the Serial Port
----------------------------------------------------------------------------*/
int LockSerPort()
{
    FILE *fdserlck;
    long rPID;
    int bRead, bWrite, lckCNT;
    char cmdFile[256];
    char command[255];
    char *SubStrPos = NULL;

    if(bVerbose) fprintf(stderr, "\nAttempting to get lock on Serial Port %s...  ",szttyDevice);

    if (! bConnect) {
        if(bVerbose) fprintf(stderr, "LockSerPort: connection not requested\n");
        return(2);
    }

    fdserlck = fopen(devLCKfile, "a");
    if (fdserlck == NULL) {
	if(bVerbose) fprintf(stderr, "\n");
	fprintf(stderr, "%s: %s: \nProblem locking serial device, can't open lock file: %s for write.\n\n",RunTime,ProgramName,devLCKfile);
	exit(2);
    }
    bWrite = fprintf(fdserlck, "%ld\n", PID);
    if (bWrite <= 0) fprintf(stderr, "\n WARNING: Problem writing lock file\n");
    fclose(fdserlck);

    rPID = 0;
    lckCNT = -1;
    while(rPID != PID && lckCNT++ < yLockWait) {
	fdserlck = fopen(devLCKfile, "r");
	if (fdserlck == NULL) {
	    if(bVerbose) fprintf(stderr, "\n");
	    fprintf(stderr, "%s: %s: \nProblem locking serial device, can't open lock file: %s for read.\n\n",RunTime,ProgramName,devLCKfile);
	    exit(2);
	}
	bRead = fscanf(fdserlck, "%ld", &rPID);
        if (bRead <= 0 && bRead != EOF) fprintf(stderr, "\n WARNING: Problem writing lock file\n");
	fclose(fdserlck);
	sprintf(cmdFile,"/proc/%ld/cmdline",rPID);
	fdserlck = fopen(cmdFile, "r");
	if (fdserlck != NULL) {
	    bRead = fscanf(fdserlck, "%s", command);
            if (bRead <= 0 && bRead != EOF) fprintf(stderr, "\n WARNING: Problem writing lock file\n");
	    fclose(fdserlck);
	    SubStrPos = strstr(command, ProgramName);
	} else
	    SubStrPos = NULL;
	if(bVerbose) fprintf (stderr, "\nrPID: %ld SubStrPos: (%s) command: %s",rPID,SubStrPos,command);
	if (rPID != PID) {
	     if (SubStrPos == NULL) {
		 if(bVerbose) fprintf (stderr, "\n");
		 fprintf(stderr, "%s: %s: Clearing stale serial port lock. (%ld)\n",RunTime,ProgramName,rPID);
		 ClrSerLock(rPID);
	     } else if (yLockWait > 0)
	         Delay(1, 0L);
	}
    }
    if (bVerbose && rPID == PID) fprintf(stderr, " Appears we got the lock.\n");
    if (rPID != PID) {
	ClrSerLock(PID);
	if(bVerbose) fprintf (stderr, "\n");
	fprintf(stderr, "%s: %s: Problem locking serial device %s, couldn't get the lock for %ld, locked by %ld.\n\n",RunTime,ProgramName,szttyDevice,PID,rPID);
	exit(2);
    }
    return TRUE;
}


#ifndef noMySQL
/*--------------------------------------------------------------------------
        GetParmsSQL
        Reads command line parameters.
----------------------------------------------------------------------------*/
int GetParmsSQL(int argc, char *argv[])
{
    extern char *optarg;
    extern int optind, opterr, optopt;
    int c;
    char myOPTS[13] = "";

    if(argc == 1)
        return 0;                               /* no parms at all */

    myOPTS[0] = '\0';
    strcpy(myOPTS, "D:H:P:p:S:u:");

     optind = 1;
     while ((c = getopt_long(argc, argv, myOPTS, longopts, NULL )) != EOF) {
        switch (c) {
            case 'H':
                opt_host_name_s = (char *)optarg;
                break;
            case 'P':
                SQLport = atoi(optarg);
                break;
            case 'S':
                opt_socket_name_s = (char *)optarg;
                break;
            case 'D':
                opt_db_name_s = (char *)optarg;
                bUseSQL = TRUE;
                break;
            case 'u':
                opt_user_name_s = (char *)optarg;
                break;
            case 'p':
                opt_password_s = getMemPtr(strlen(optarg));
                strcpy(opt_password_s, optarg);
                break;
            case '?': /* user entered unknown option */
            case ':': /* user entered option without required value */
                return 0;

            default:
                break;
        }
    }
    return -1;
}
#endif

/*--------------------------------------------------------------------------
	GetParms
	Reads command line parameters.
----------------------------------------------------------------------------*/
int GetParms(int argc, char *argv[])
{
    extern char *optarg;
    extern int optind, opterr, optopt;
    int c, i;
    char *pos;
    char myOPTS[80] = "";
    FILE *fd;
    char *fBuf = NULL;
    int fLen = 0;
    char *SubStrPos = NULL;
    char *token;

    /* Set command line defaults */
    *szttyDevice = '\0';
    yDelay = 10;

    if(argc == 1)
        return 0;				/* no parms at all */

    myOPTS[0] = '\0';
//        2A:a:b:C:cD:d:EeF:fG:gH:hIi:K:k:L:lMmNOo:P:p:Qq:RrS:sTtU:u:VvW:w:X:xYyZ:z
// avail       B(resv)                J 
    strcpy(myOPTS, "2A:a:b:C:cd:eF:fG:ghIK:k:L:lMmNnORrsTtU:vW:w:X:xYyZ:z");	/* b:? */
#ifndef noMySQL
    strcat(myOPTS, "D:EH:i:j:o:P:p:Qq:S:u:V");
#endif
    while ((c = getopt_long(argc, argv, myOPTS, longopts, NULL )) != EOF) {
        switch (c) {
            case '2': bVpro2       = TRUE; break; 
            case 'n': bBKLOn       = bConnect = TRUE; break;
            case 'f': bBKLOff      = bConnect = TRUE; break;
            case 'r': bVer         = bConnect = TRUE; break;
            case 'm': bModel       = bConnect = TRUE; break;
            case 'x': bGetRTD      = bConnect = TRUE; break;
            case 'l': bGetHLD      = bConnect = TRUE; break;
            case 'g': bGetGD       = bConnect = TRUE; break;
            case 't': bGetTime     = bConnect = TRUE; break;
            case 's': bSetTime     = bConnect = TRUE; break;
            case 'T': bCorrectTime = bConnect = TRUE; break;
            case 'R': bVPwVer      = TRUE; break;
            case 'v': bVerbose     = TRUE; break;
            case 'M': bMtime       = TRUE; break;
            case 'h': bHelp        = TRUE; break;
            case 'y': bWUresponse  = TRUE; break;
            case 'Y': bWUsuppress  = TRUE; break;
            case 'c': bZcommas     = TRUE; break;
            case 'z': bListCols    = TRUE; break;
            case 'I': bTryTwice    = TRUE; break;
            case 'e': bSwapEndian  = TRUE; break;
            case 'O': bSTDOUT      = TRUE; break;
#ifndef noMySQL
            case 'E': bEnableMYSQL = bUseSQL = bMtime = TRUE; break;
            case 'Q': bQuiet       = TRUE; break;
            case 'V': bVerboseSQL  = TRUE; break;
#endif
            case 'C':
                bMetric = TRUE;
                if (optarg != NULL) {
                    if (strcmp(optarg, "tipmod") == 0 || strcmp(optarg, "1") == 0)
                        bTipMod = TRUE;
                    else if (strcmp(optarg, "0") != 0) {
                        fprintf(stderr, "\nvproweather: Illegal -C value specified (%s).\n",optarg);
                        return 0;
                    }
                }
                break;
            case 'a':
                if (optarg != NULL) 
                    i = atoi(optarg);
                else
                    i = 512;
                if (i < 1 || i > 513) {
                    fprintf(stderr, "\nvproweather: Illegal page count specified (%i).\n",i);
                    return 0;
                }
                bGetDMP =  TRUE;
                bConnect = TRUE;
                wDMPPages = i;
                break;
            case 'A':
                if (strlen(optarg) > sizeof(sGetDMPAFT)-1) {
                    fprintf(stderr, "\nvproweather: Archive data date string too long max %lu\n",(long unsigned)sizeof(sGetDMPAFT)-1);
                    return 0;
                }
                strcpy(sGetDMPAFT, optarg);
                bGetDMP =  TRUE;
                bConnect = TRUE;
                break;
            case 'd':
                /* Get delay time */
                i = atoi(optarg);
                if (i < 0 || i > 255) {
                    fprintf(stderr, "\nvproweather: Illegal delay specified (%i),\n",i);
                    return 0;
                }
                yDelay = (unsigned char)i;
                break;
            case 'w':
                /* get number of wakeup attempts to make */
                i = atoi(optarg);
                if (i < 2) {
                    fprintf(stderr, "\nvproweather: Illegal wakeup attempts specified (%i), must be at least 2.\n",i);
                    return 0;
                }
                yWattempts = i;
                break;
            case 'W':
                /* get read character wait time */
                i = atoi(optarg);
                if (i < 250 || i > 5000) {
                    fprintf(stderr, "\nvproweather: Illegal read character wait time specified (%i), must be between 250 & 5000 uS.\n",i);
                    return 0;
                }
                yReadCharWait = i * 1000;
                break;
            case 'X':
                if (strlen(optarg) > sizeof(sloopDumpFN)-1) {
                    fprintf(stderr, "\nvproweather: Real Time raw data filename too long max %lu\n",(long unsigned)sizeof(sloopDumpFN)-1);
                    return 0;
                }
                strcpy(sloopDumpFN, optarg);
                break;
            case 'L':
                if (strlen(optarg) > sizeof(shiloDumpFN)-1) {
                    fprintf(stderr, "\nvproweather: High/Low raw data filename too long max %lu\n",(long unsigned)sizeof(shiloDumpFN)-1);
                    return 0;
                }
                strcpy(shiloDumpFN, optarg);
                break;
            case 'G':
                if (strlen(optarg) > sizeof(sgraphDumpFN)-1) {
                    fprintf(stderr, "\nvproweather: Graph raw data filename too long max %lu\n",(long unsigned)sizeof(sgraphDumpFN)-1);
                    return 0;
                }
                strcpy(sgraphDumpFN, optarg);
                break;
            case 'U':
                if (strlen(optarg) > sizeof(sloopUseDumpFN)-1) {
                    fprintf(stderr, "\nvproweather: Input Graph raw data filename too long max %lu\n",(long unsigned)sizeof(sloopUseDumpFN)-1);
                    return 0;
                }
                strcpy(sloopUseDumpFN, optarg);
                break;
            case 'Z':
                if (strlen(optarg) > sizeof(sDataFN)-1) {
                    fprintf(stderr, "\nvproweather: Output data filename too long max %lu\n",(long unsigned)sizeof(sDataFN)-1);
                    return 0;
                }
                strcpy(sDataFN, optarg);
                break;
            case 'B':
                if (strlen(optarg) > sizeof(sEnvList)-1) {
                    fprintf(stderr, "\nvproweather: ENV data input filename too long max %lu\n",(long unsigned)sizeof(sEnvList)-1);
                    return 0;
                }
                strcpy(sEnvList, optarg);
                break;
            case 'N':
                bCalcSun    = TRUE;
                if (optarg != NULL && strcmp(optarg, "showsec") == 0) bSolarNoonSec = TRUE;
                break;
            case 'K':
                sscanf(optarg, "%lf", &fLatitude);
                break;
            case 'k':
                sscanf(optarg, "%lf", &fLongitude);
                break;
            case 'F':
                if (strcmp(optarg, "LIST") == 0) {
                    ListRTDataFmt();
                    exit(0);
                }
                pos = strrchr(optarg, '}');
                if (pos != NULL) {
                    sFormat = getMemPtr(strlen(optarg)+2);
                    if (!sFormat) {
                        perror("vproweather: Problem allocating memory for format");
                        exit(2);
                    }
                    strcpy(sFormat, optarg);
                    strcat(sFormat, " ");	/* need on pad space for command line ientry */
                } else {
                    errno = 0;
                    fd = fopen(optarg, "r");
                    if (errno != 0 || fd < 0) {
                        fprintf (stderr, "\nvproweather: Problem opening format file \"%s\", %s.\n\n",optarg,strerror (errno));
                        exit(2);
                    }
                    while (fgetc(fd) != EOF) fLen++;
                    sFormat = getMemPtr(fLen+1);
                    if (!sFormat) {
                        fclose(fd);
                        perror("vproweather: Problem allocating memory for format");
                        exit(2);
                    }
                    rewind(fd);
                    fBuf = sFormat;
                    while ((*fBuf++ = fgetc(fd)) != EOF);
                    fclose(fd);
                    *--fBuf = '\0';
                }
                bGetRTD = bConnect = TRUE;
                break;
#ifndef noMySQL
            case 'q':
                sProfileDataFN = getMemPtr(sizeof(optarg));
                strcpy(sProfileDataFN, optarg);
                bProfiling = TRUE;
                break;
            case 'i':
                SubStrPos = strstr(optarg, ":");
                if (SubStrPos == NULL) {
                    sImportDataFN = getMemPtr(sizeof(optarg));
                    strcpy(sImportDataFN, optarg);
                } else {
                    token = strtok (optarg,":");
                    sImportDataFN = getMemPtr(sizeof(optarg));
                    strcpy(sImportDataFN, token);
                    token = strtok (NULL, ":");
                    if (token != NULL)
                        if (strcmp(token, "OVERRIDE") == 0) bImportOR = TRUE;
                }
                bImport = TRUE;
                break;
            case 'o':
                SubStrPos = strstr(optarg, ":");
                if (SubStrPos == NULL) {
                    sExportDataFN = getMemPtr(sizeof(optarg));
                    strcpy(sExportDataFN, optarg);
                } else {
                    token = strtok (optarg,":");
                    sExportDataFN = getMemPtr(sizeof(token));
                    strcpy(sExportDataFN, token);
                    token = strtok (NULL, ":");
                    if (token != NULL) {
                        sExportBegDate = getMemPtr(sizeof(token));
                        strcpy(sExportBegDate, token);
                        token = strtok (NULL, ":");
                        if (token != NULL) {
                            sExportEndDate = getMemPtr(sizeof(token));
                            strcpy(sExportEndDate, token);
                        }
                    }
                }
                bExport = TRUE;
                break;
            case 'H':
                opt_host_name_s = (char *)optarg;
                break;
            case 'P':
                SQLport = atoi(optarg);
                break;
            case 'S':
                opt_socket_name_s = (char *)optarg;
                break;
            case 'D':
                opt_db_name_s = (char *)optarg;
                bUseSQL = TRUE;
                break;
            case 'u':
                opt_user_name_s = (char *)optarg;
                break;
            case 'p':
                if (optarg != NULL) {
                    opt_password_s = getMemPtr(strlen(optarg));
                    strcpy(opt_password_s, optarg);
                } else
                    opt_password_s = getPW();
                break;
            case 'j':
                /* Get Console ID */
                i = atoi(optarg);
                if (i < 0 || i > 999) {
                    fprintf(stderr, "\nvproweather: Illegal Console ID specified (%i),\n",i);
                    return 0;
                }
                ConsoleID = i;
                bConsoleIDset = TRUE;
                break;
#endif
            case '?': /* user entered unknown option */
            case ':': /* user entered option without required value */
                return 0;
            
            default:
                break;
        }
    }
    if (argv[optind] != NULL) strcpy(szttyDevice, argv[optind]);
    if (bCalcSun && ((latitude ==  -1000.0 && fLatitude == -1000.0) || (longitude ==  -1000.0 && fLongitude == -1000.0))) bConnectEE = TRUE;
    if ((bConnect || bConnectEE) && strlen(szttyDevice) <= 0) {
        if (! bHelp && ! bVPwVer) {
            fprintf(stderr, "\nvproweather: No serial device specified");
            if (bCalcSun && ((latitude ==  -1000.0 && fLatitude == -1000.0) || (longitude ==  -1000.0 && fLongitude == -1000.0))) fprintf(stderr, ". Needed: Lat and/or Lon not known for -N.");
            fprintf(stderr, "\n");
        }
        if (bVPwVer) return -1;
        return 0;
    }
    if (bWUresponse && bWUsuppress) {
        fprintf(stderr, "\nvproweather: -y and -Y are mutually exclusive.\n");
        return 0;
    }
    if (bGetRTD && strlen(sloopUseDumpFN) > 0) {
        fprintf(stderr, "\nvproweather: -x and -U are mutually exclusive.\n");
        return 0;
    }
#ifndef noMySQL
    if (! bEnableMYSQL) bUseSQL = FALSE;
#endif
    pos = strrchr(szttyDevice, '/');
    if (pos > 0) {
        pos++;
        strcat(devLCKfile, pos);
        sprintf(devLCKfileNew,"%s.%ld",devLCKfile,PID);
    } else {
        *devLCKfile = '\0';
    }
    if(bVerbose && bConnect) fprintf(stderr, "\nszttyDevice: %s\ndevLCKfile: <%s>\ndevLCKfileNew: <%s>\n",szttyDevice,devLCKfile,devLCKfileNew);

    if (! bVpro2)
        setupVP1();
    else
        setupVP2();

    return -1;
}


/*--------------------------------------------------------------------------
	WakeUp
	Wakes up the weather station per the Davis specs.
----------------------------------------------------------------------------*/
int WakeUp(int nfd, char wch)
{
	char ch;
	int i;
	int RC;

	if(bVerbose) fprintf(stderr, "\nGoing to make %i attempts to wake it up",yWattempts);
	for(i = 0; i < yWattempts; i++)
	{
		if(bVerbose) fprintf(stderr, "\nAttempt %i Flushing all I/O ",i+1);
		tcflush(nfd, TCIOFLUSH);
		ch = wch;
		if(bVerbose) fprintf(stderr, "\nSending 0x%02x ",ch);
		errno = 0;
		RC = write(nfd, &ch, 1);		/* send wake up char */
                if (RC <= 0 && i > 2) fprintf(stderr, "\n WARNING: WakeUp Problem sending command\n");
		if (errno != 0)
			fprintf (stderr, "\nvproweather: Problem writing serial device, %s.\n",strerror (errno));
		if(bVerbose) fprintf(stderr, "RC %i. ",RC);
		if(bVerbose) fprintf(stderr, "Waiting for xmit to complete. ");
		RC = tcdrain(nfd);
		if(bVerbose) {
			fprintf(stderr, "Completed. RC %i.\nReading ",RC);
			if (i >= 0 ) fprintf(stderr, "waiting for up to %i mS. ",250*(i+1));
		}
		RC = ReadNextCharTO(nfd, &ch, 250000*(i+1)); 		/* read a char */
		if(bVerbose) fprintf(stderr, "RC %i. ",RC);
		if (ch == VP_LF) {
			if(bVerbose) fprintf(stderr, "Got 0x%02x, getting 0x%02x. ",ch,VP_CR);
			RC = ReadNextCharTO(nfd, &ch, 250000*(i+1));
			if(bVerbose) fprintf(stderr, "RC %i. ",RC);
			if (i > 0 && ch == VP_LF && ! bWUresponse && ! bWUsuppress) 
			    fprintf(stderr, "Expecting 0x%02x, got 0x%02x. Use -y to accept this as a valid acknowledgment, -Y to suppress this message without accepting this as a valid acknowledgment.\n",VP_CR,ch);
			if (ch == VP_CR || (i > 0 && ch == VP_LF && bWUresponse)) {
				if (bVerbose) {
					if (ch == VP_LF) 
						fprintf(stderr, "Expecting 0x%02x, got 0x%02x, continuing anyway.\n",VP_CR,ch);
					else {
						fprintf(stderr, "Got 0x%02x.\n",ch);
					}
						fprintf(stderr, "Weather station woke up after %d retries\n\n",i+1);
				}
				return -1;			/* okay, its awake now */
			} else if (bVerbose)
				fprintf(stderr, "Didn't get 0x%02x got 0x%02x\n",VP_CR,ch);
		} else if (bVerbose)
			fprintf(stderr, "Didn't get 0x%02x got 0x%02x\n",VP_LF,ch);
	}

	return 0;					/* failed to wake up */	
}


/*--------------------------------------------------------------------------
	ReadNextChar
	Reads the next character from the serial device. Returns zero if no
	character was available.
----------------------------------------------------------------------------*/
int ReadNextChar(int nfd, char *pChar)
{
	int nResult;

	memset (pChar, 0, sizeof (*pChar));
	errno = 0;
	nResult = read(nfd, pChar, 1);
	if (errno != 0)
		fprintf (stderr, "vproweather: Problem reading serial device, %s.\n",strerror (errno));
	if(nResult == -1) {
		if (errno == 0) perror("vproweather: Problem reading serial device.");
                ClrSerLock(PID);
		exit(2);
	}
	return nResult;
}


/*--------------------------------------------------------------------------
	ReadNextCharTO
	Reads the next character from the serial device with timout. Returns 
	zero if no character was available.
----------------------------------------------------------------------------*/
int ReadNextCharTO(int nfd, char *pChar, int timeout)
{
	int nResult = -1;
	int errnoS = 0;
	long int eUsecs, sSecs, cSecs, sUsecs, cUsecs;
	struct timeval tv;

	errno = 0;
	eUsecs = sSecs = cSecs = sUsecs = cUsecs = 0;
	memset (pChar, 0, sizeof (*pChar));

	if (yReadCharWait > 0) timeout = yReadCharWait;
	if (timeout > 0) {
		eUsecs = 0;
		gettimeofday(&tv, NULL); 
		sSecs = tv.tv_sec;
		sUsecs = tv.tv_usec;
		while (nResult <= 0 && (errno == 0 || errno == EAGAIN) && eUsecs <= timeout) { 
			errno = 0;
			nResult = read(nfd, pChar, 1);
			errnoS = errno;
			gettimeofday(&tv, NULL);
			cSecs = tv.tv_sec;
			cUsecs = tv.tv_usec;
			eUsecs = ((cSecs-sSecs)*1000000) + (cUsecs-sUsecs);
		}
	} else {
		errno = 0;
		nResult = read(nfd, pChar, 1);
		errnoS = errno;
	}
	if (errnoS != 0 && errnoS != EAGAIN)
		fprintf (stderr, "vproweather: (TO: errno %i) Problem reading serial device, %s.\n",errno,strerror (errnoS));
	if(nResult == -1) {
		if (errnoS == 0) perror("vproweather: (TO) Problem reading serial device.\n");
		if (errnoS != EAGAIN) {
			ClrSerLock(PID);
			exit(2);
		}
	}
	if (errnoS == EAGAIN) 
		return 0;
	else
		return nResult; 
}


/*--------------------------------------------------------------------------
	Delay
	Delays by the number of seconds and microseconds.
----------------------------------------------------------------------------*/
void Delay(int secs, long microsecs)
{
	static struct timeval t1;

	t1.tv_sec = (long)secs;
	t1.tv_usec = microsecs;
	if ( select(0, (fd_set *)0, (fd_set *)0, (fd_set *)0, &t1)  < 0 )
		 perror("Internal error: error in select()");
	return;
} 


/*--------------------------------------------------------------------------
	ReadToBuffer
	Reads data to a buffer until no more characters are available. If
	the buffer overflows, returns -1. Otherwise, returns the number of
	characters read.
----------------------------------------------------------------------------*/
int ReadToBuffer(int nfd, char *pszBuffer, int nBufSize)
{
	int nPos = 0;				/* current character position */
	char *pBuf = pszBuffer;

	while(nPos < nBufSize) {
		if(!ReadNextCharTO(nfd, pBuf++, 250000))
			return nPos;		/* no character available */ 
		++nPos;
	}		
	return -1;				/* buffer overflow */
}

